# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2016-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing topic model predictions creation

"""
from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)

import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.basic_topic_model_steps as topic_pred

def setup_module():
    """Setup for the module

    """
    common_setup_module()
    test = TestTopicModel()
    test.bigml = {"method": "setup_scenario02"}
    test.setup_scenario02()


def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()


class TestTopicModel:
    """Testing topic model commands"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def test_scenario01(self):
        """
        Scenario: Successfully building topic model test distributions from dataset:
            Given I created the dataset in setup_scenario02
            And I create BigML topic model resources from dataset to test "<test>" with options "<options>" and log predictions in "<output>"
            And I check that the topic model has been created
            And I check that the topic distributions are ready
            Then the local topic distribution file is like "<topic_distribution_file>"
        """
        print(self.test_scenario01.__doc__)
        headers = ["data", "options", "output", "topic_distribution_file"]
        examples = [
            ['data/spam.csv', '--test-separator="\t" --prediction-header',
             'scenario1_td/topic_distributions.csv',
             'check_files/topic_distributions_spam.csv',
             './check_files/topic_distributions_spam.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            topic_pred.i_create_all_td_resources_from_dataset(
                self, example["data"], example["options"], example["output"])
            topic_pred.i_check_create_topic_model(self)
            topic_pred.i_check_create_topic_distributions(self)
            topic_pred.i_check_topic_distributions(
                self, example["topic_distribution_file"])

    def setup_scenario02(self):
        """
        Scenario: Successfully building text source from local file:
            Given I create BigML dataset uploading train "<data>" file with attributes "<attributes>" in "<output>"
            And I check that the source has been created
            Then I check that the dataset has been created
        """
        print(self.setup_scenario02.__doc__)
        headers = ["data", "attributes", "output"]
        examples = [
            ['data/spam.csv', 'data/spam_attributes.json',
             'scenario2_td/topic_distributions.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_dataset_with_attributes( \
                self, data=example["data"], attributes=example["attributes"],
                output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)

    def test_scenario03(self):
        """
        Scenario: Successfully building test predictions from source
            Given I created the dataset in setup_scenario02
            And I create BigML topic model resources from source to test "<test>" with options "<options>" and log predictions in "<output>"
            And I check that the dataset has been created
            And I check that the topic model has been created
            And I check that the topic distributions are ready
            Then the local topic distribution file is like "<topic_distribution_file>"
        """
        print(self.test_scenario03.__doc__)
        headers = ["data", "options", "output", "topic_distribution_file"]
        examples = [
            ['data/spam.csv', '--test-separator="\t" --prediction-header',
             'scenario3_td/topic_distributions.csv',
             'check_files/topic_distributions_spam.csv',
             './check_files/topic_distributions_spam.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            topic_pred.i_create_all_td_resources_from_source(
                self, example["data"], example["options"], example["output"])
            test_pred.i_check_create_dataset(self, suffix=None)
            topic_pred.i_check_create_topic_model(self)
            topic_pred.i_check_create_topic_distributions(self)
            topic_pred.i_check_topic_distributions(
                self, example["topic_distribution_file"])

    def test_scenario04(self):
        """
        Scenario: Successfully building test predictions from topic model
            Given I created the dataset in setup_scenario02
            And I create topic model from dataset
            And I create BigML topic model resources from model to test "<test>" with options "<options>" and log predictions in "<output>"
            And I check that the dataset has been created
            And I check that the topic distributions are ready
            Then the local topic distribution file is like "<topic_distribution_file>"
        """
        print(self.test_scenario04.__doc__)
        headers = ["data", "options", "output", "topic_distribution_file"]
        examples = [
            ['data/spam.csv', '--test-separator="\t" --prediction-header',
             'scenario4_td/topic_distributions.csv',
             'check_files/topic_distributions_spam.csv',
             './check_files/topic_distributions_spam.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            topic_pred.i_create_topic_model_from_dataset(
                self, example["output"])
            topic_pred.i_check_create_topic_model(self)
            topic_pred.i_create_all_td_resources_from_model(
                self, example["data"], example["options"], example["output"])
            topic_pred.i_check_create_topic_distributions(self)
            topic_pred.i_check_topic_distributions(
                self, example["topic_distribution_file"])


    def test_scenario05(self):
        """
        Scenario: Successfully building batch topic distributions from topic model
            Given I created the dataset in setup_scenario02
            And I create BigML batch topic distribution resources from dataset to test "<test>" and log predictions in "<output>"
            And I check that the topic model has been created
            And I check that the batch topic distribution is ready
            Then the local topic distribution file is like "<predictions_file>"

            Examples:
            | test                    | options                  | output                                   |topic_distribution_file           |
            | ../data/spam.csv        | --test-separator="\t --prediction-header" | ./scenario5_td/topic_distributions.csv   | ./check_files/topic_distributions_spam.csv   |


        """
        print(self.test_scenario05.__doc__)
        headers = ["data", "options", "output", "topic_distribution_file"]
        examples = [
            ['data/spam.csv', '--test-separator="\t" --prediction-header',
             'scenario5_td/topic_distributions.csv',
             'check_files/topic_distributions_spam.csv',
             './check_files/topic_distributions_spam.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            topic_pred.i_create_topic_model_from_dataset(
                self, example["output"])
            topic_pred.i_check_create_topic_model(self)
            topic_pred.i_create_topic_distribution_from_model_remote(
                self, test=example["data"], options=example["options"],
                output=example["output"])
            topic_pred.i_check_create_topic_distributions(self)
            # the check between local and remote predictions is removed till we
            # add new options in the local side to match the remote ones
            # topic_pred.i_check_topic_distributions(self, example[3])



# -*- coding: utf-8 -*-
#
# Copyright 2020-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Resources management functions

"""


import sys

import bigml.api

from bigmler.utils import (dated, get_url, log_message, plural, check_resource,
                           check_resource_error, log_created_resources,
                           is_shared)
from bigmler.reports import report

from bigmler.resourcesapi.common import SEED, \
    FIELDS_QS, ALL_FIELDS_QS
from bigmler.resourcesapi.common import set_basic_model_args, \
    configure_input_fields, update_sample_parameters_args, update_json_args, \
    wait_for_available_tasks

def set_anomaly_args(args, name=None, fields=None, anomaly_fields=None):
    """Return anomaly arguments dict

    """
    if name is None:
        name = args.name
    if anomaly_fields is None:
        anomaly_fields = args.anomaly_fields_

    anomaly_args = set_basic_model_args(args, name)
    anomaly_args.update({
        "seed": SEED if args.seed is None else args.seed,
        "anomaly_seed": (SEED if args.anomaly_seed is None
                         else args.anomaly_seed)
    })

    if anomaly_fields and fields is not None:
        input_fields = configure_input_fields(fields, anomaly_fields)
        anomaly_args.update(input_fields=input_fields)
    if args.id_fields is not None:
        anomaly_args.update({"id_fields": args.id_fields_})
    if args.top_n > 0:
        anomaly_args.update(top_n=args.top_n)
    if args.forest_size > 0:
        anomaly_args.update(forest_size=args.forest_size)

    anomaly_args = update_sample_parameters_args(anomaly_args, args)

    if 'anomaly' in args.json_args:
        update_json_args(anomaly_args, args.json_args.get('anomaly'), fields)

    return anomaly_args


def set_publish_anomaly_args(args):
    """Set args to publish anomaly

    """
    public_anomaly = {}
    if args.public_anomaly:
        public_anomaly = {"private": False}
        if args.model_price:
            public_anomaly.update(price=args.model_price)
        if args.cpp:
            public_anomaly.update(credits_per_prediction=args.cpp)
    return public_anomaly


def create_anomalies(datasets, anomaly_ids, anomaly_args,
                     args, api=None, path=None,
                     session_file=None, log=None):
    """Create remote anomalies

    """
    if api is None:
        api = bigml.api.BigML()

    anomalies = anomaly_ids[:]
    existing_anomalies = len(anomalies)
    anomaly_args_list = []
    datasets = datasets[existing_anomalies:]
    # if resuming and all anomalies were created,
    # there will be no datasets left
    if datasets:
        if isinstance(anomaly_args, list):
            anomaly_args_list = anomaly_args

        # Only one anomaly per command, at present
        number_of_anomalies = 1
        message = dated("Creating %s.\n" %
                        plural("anomaly detector", number_of_anomalies))
        log_message(message, log_file=session_file,
                    console=args.verbosity)

        query_string = FIELDS_QS
        inprogress = []
        for i in range(0, number_of_anomalies):
            wait_for_available_tasks(inprogress, args.max_parallel_anomalies,
                                     api, "anomaly")
            if anomaly_args_list:
                anomaly_args = anomaly_args_list[i]

            anomaly = api.create_anomaly(datasets, anomaly_args, retries=None)
            anomaly_id = check_resource_error(anomaly,
                                              "Failed to create anomaly: ")
            log_message("%s\n" % anomaly_id, log_file=log)
            anomaly_ids.append(anomaly_id)
            inprogress.append(anomaly_id)
            anomalies.append(anomaly)
            log_created_resources("anomalies", path, anomaly_id, mode='a')

        if args.verbosity:
            if bigml.api.get_status(anomaly)['code'] != bigml.api.FINISHED:
                try:
                    anomaly = check_resource(anomaly, api.get_anomaly,
                                             query_string=query_string,
                                             raise_on_error=True)
                except Exception as exception:
                    sys.exit("Failed to get a finished anomaly: %s" %
                             str(exception))
                anomalies[0] = anomaly
            message = dated("Anomaly created: %s\n" %
                            get_url(anomaly))
            log_message(message, log_file=session_file,
                        console=args.verbosity)
            if args.reports:
                report(args.reports, path, anomaly)

    return anomalies, anomaly_ids


def get_anomalies(anomaly_ids, args, api=None, session_file=None):
    """Retrieves remote anomalies in its actual status

    """
    if api is None:
        api = bigml.api.BigML()
    anomaly_id = ""
    anomalies = anomaly_ids
    anomaly_id = anomaly_ids[0]
    message = dated("Retrieving %s. %s\n" %
                    (plural("anomaly detector", len(anomaly_ids)),
                     get_url(anomaly_id)))
    log_message(message, log_file=session_file, console=args.verbosity)
    # only one anomaly to predict at present
    try:
        # we need the whole fields structure when exporting fields
        query_string = FIELDS_QS if not args.export_fields else ALL_FIELDS_QS
        anomaly = check_resource(anomaly_ids[0], api.get_anomaly,
                                 query_string=query_string,
                                 raise_on_error=True)
    except Exception as exception:
        sys.exit("Failed to get a finished anomaly: %s" % str(exception))
    anomalies[0] = anomaly

    return anomalies, anomaly_ids


def update_anomaly(anomaly, anomaly_args, args,
                   api=None, path=None, session_file=None):
    """Updates anomaly properties

    """
    if api is None:
        api = bigml.api.BigML()
    message = dated("Updating anomaly detector. %s\n" %
                    get_url(anomaly))
    log_message(message, log_file=session_file,
                console=args.verbosity)
    anomaly = api.update_anomaly(anomaly, anomaly_args)
    check_resource_error(anomaly, "Failed to update anomaly: %s"
                         % anomaly['resource'])
    anomaly = check_resource(anomaly, api.get_anomaly,
                             query_string=FIELDS_QS,
                             raise_on_error=True)
    if is_shared(anomaly):
        message = dated("Shared anomaly link. %s\n" %
                        get_url(anomaly, shared=True))
        log_message(message, log_file=session_file, console=args.verbosity)
        if args.reports:
            report(args.reports, path, anomaly)

    return anomaly



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2016-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing linear regression predictions creation

"""


from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)


import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.basic_batch_tst_prediction_steps as batch_pred
import bigmler.tests.basic_linear_r_steps as lr_pred

def setup_module():
    """Setup for the module

    """
    common_setup_module()
    test = TestPrediction()
    test.bigml = {"method": "setup_scenario02"}
    test.setup_scenario02()


def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()


class TestPrediction:
    """Testing linear regression predictions"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def test_scenario01(self):
        """
        Scenario: Successfully building linear regression test predictions from start with no headers:
            Given I create BigML linear regression resources uploading train "<data>" file with no headers to test "<test>" with no headers and log predictions in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the linear regression model has been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario01.__doc__)
        headers = ["data", "test", "output", "predictions_file"]
        examples = [
            ['data/grades_nh.csv', 'data/test_grades_nh.csv',
             'scenario1_lrr_nh/predictions.csv', 'check_files/predictions_grades_lrr.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            lr_pred.i_create_all_lr_resources_with_no_headers(
                self, example["data"], example["test"], example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            lr_pred.i_check_create_lr_model(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def setup_scenario02(self):
        """
        Scenario: Successfully building test predictions from start:
            Given I create BigML linear regression resources uploading train "<data>" file to test "<test>" and log predictions in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the model has been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.setup_scenario02.__doc__)
        headers = ["data", "test", "output", "predictions_file"]
        examples = [
            ['data/grades.csv', 'data/test_grades_no_missings.csv',
             'scenario1_lrr/predictions.csv',
             'check_files/predictions_grades_lrr.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            lr_pred.i_create_all_lr_resources(
                self, example["data"], example["test"], example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            lr_pred.i_check_create_lr_model(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario03(self):
        """
        Scenario: Successfully building test predictions from source
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML linear regression resources using source to test "<test>" and log predictions in "<output>"
            And I check that the dataset has been created
            And I check that the model has been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario03.__doc__)
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario1_lrr', '{"data": "data/grades.csv",' +
             ' "output": "scenario1_lrr/predictions.csv",' +
             ' "test": "data/test_grades_no_missings.csv"}',
             'data/test_grades_no_missings.csv',
             'scenario2_lrr/predictions.csv',
             'check_files/predictions_grades_lrr.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            lr_pred.i_create_lr_resources_from_source(
                self, None, test=example["test"], output=example["output"])
            test_pred.i_check_create_dataset(self, suffix=None)
            lr_pred.i_check_create_lr_model(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario04(self):
        """
        Scenario: Successfully building test predictions from dataset
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML linear regression resources using dataset to test "<test>" and log predictions in "<output>"
            And I check that the model has been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario04.__doc__)
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario1_lrr', '{"data": "data/grades.csv",' +
             ' "output": "scenario1_lrr/predictions.csv",' +
             ' "test": "data/test_grades_no_missings.csv"}',
             'data/test_grades_no_missings.csv',
             'scenario3_lrr/predictions.csv',
             'check_files/predictions_grades_lrr.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            lr_pred.i_create_lr_resources_from_dataset(
                self, None, test=example["test"], output=example["output"])
            lr_pred.i_check_create_lr_model(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario05(self):
        """
        Scenario: Successfully building test predictions from model
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML linear regression resources using model to test "<test>" and log predictions in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario05.__doc__)
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario1_lrr', '{"data": "data/grades.csv",' +
             ' "output": "scenario1_lrr/predictions.csv",' +
             ' "test": "data/test_grades_no_missings.csv"}',
             'data/test_grades_no_missings.csv',
             'scenario4_lrr/predictions.csv',
             'check_files/predictions_grades_lrr.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            lr_pred.i_create_lr_resources_from_model(
                self, test=example["test"], output=example["output"])
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario06(self):
        """
        Scenario: Successfully building batch test predictions from model
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML linear regression resources using model to test "<test>" as a batch prediction and log predictions in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario06.__doc__)
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario1_lrr', '{"data": "data/grades.csv",' +
             ' "output": "scenario1_lrr/predictions.csv",' +
             ' "test": "data/test_grades_no_missings.csv"}',
             'data/test_grades_no_missings.csv',
             'scenario5_lrr/predictions.csv',
             'check_files/predictions_grades_lrr.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            lr_pred.i_create_lr_resources_from_model_remote(
                self, test=example["test"], output=example["output"])
            batch_pred.i_check_create_batch_prediction(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,attribute-defined-outside-init
# Auxiliar class: Simple stoppable HTTPServer extracted from
#
# http://code.activestate.com/recipes/425210-simple-stoppable-server-using-socket-timeout/
#
# Unless required by applicable law or agreed to in writing, this software
# is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.

import http.server
import socket

class StoppableHTTPServer(http.server.HTTPServer):
    """Basic stoppable HTTP server to show reports"""

    def server_bind(self):
        """Binding the server """
        http.server.HTTPServer.server_bind(self)
        self.socket.settimeout(1)
        self.run = True

    def get_request(self):
        """Get method"""
        while self.run:
            try:
                sock, addr = self.socket.accept()
                sock.settimeout(None)
                return (sock, addr)
            except socket.timeout:
                pass

    def stop(self):
        """Stopper """
        self.run = False

    def serve(self):
        """Serving """
        while self.run:
            self.handle_request()



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2021 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing fusion predictions creation

"""
import os

from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)


import bigmler.tests.composite_steps as composite_create
import bigmler.tests.basic_tst_prediction_steps as source_create


def setup_module():
    """Setup for the module

    """
    common_setup_module()


def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()


class TestComposite:
    """Testing Composite Sources"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def test_scenario01(self):
        """
        Scenario: Successfully building a composite
            And I create a source from "<data>"
            And I check that the source is ready
            And I create an composite with the source
            And I check that the composite is ready
            And I check that the source is a composite component
            And I remove the source from the composite
            And I check that the composite is empty
        """
        print(self.test_scenario01.__doc__)
        headers = ["data", "output_dir"]
        examples = [
            ['data/iris.csv', './scenario42_01']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            source_create.i_create_source( \
                self, data=example["data"], output_dir=example["output_dir"])
            source_create.i_check_create_source(self)
            self.sources = [world.source["resource"]]
            composite_create.i_create_composite_from_sources( \
                self, sources=",".join(self.sources),
                output_dir=example["output_dir"])
            composite_create.i_check_create_composite(self)
            composite_create.check_sources_in_composite(self)
            composite_create.remove_sources(
                self, output_dir=example["output_dir"])
            composite_create.i_check_create_composite(self)
            self.source = self.sources[0]
            self.sources = []
            composite_create.check_sources_in_composite(self)
            source_create.check_source_exists(self, exists=True)

    def test_scenario02(self):
        """
        Scenario: Successfully building a composite and adding sources
            And I create a source from "<data>"
            And I check that the source is ready
            And I create an empty composite and add the source
            And I check that the composite is ready
            And I check that the source is a composite component
        """
        print(self.test_scenario02.__doc__)
        headers = ["data", "output_dir"]
        examples = [
            ['data/iris.csv', './scenario42_02']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            source_create.i_create_source(
                self, data=example["data"], output_dir=example["output_dir"])
            source_create.i_check_create_source(self)
            self.sources = [world.source["resource"]]
            composite_create.i_create_empty_composite_and_add_source(
                self, add_sources=",".join(self.sources),
                output_dir=example["output_dir"])
            composite_create.i_check_create_composite(self)
            composite_create.check_sources_in_composite(self)

    def test_scenario03(self):
        """
        Scenario: Successfully building a composite
            And I create a source from "<data>"
            And I check that the source is ready
            And I create an composite with the source
            And I check that the composite is ready
            And I check that the source is a composite component
            And I remove the source from the composite
            And I check that the composite is empty
        """
        print(self.test_scenario03.__doc__)
        headers = ["data", "output_dir"]
        examples = [
            ['data/iris.csv', './scenario42_03']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            source_create.i_create_source( \
                self, data=example["data"],
                output_dir=example["output_dir"])
            source_create.i_check_create_source(self)
            self.sources = [world.source["resource"]]
            composite_create.i_create_composite_from_sources(
                self, sources=",".join(self.sources),
                output_dir=example["output_dir"])
            composite_create.i_check_create_composite(self)
            composite_create.check_sources_in_composite(self)
            composite_create.delete_sources(
                self, output_dir=example["output_dir"])
            composite_create.i_check_create_composite(self)
            self.source = self.sources[0]
            self.sources = []
            composite_create.check_sources_in_composite(self)
            source_create.check_source_exists(self, exists=False)

    def test_scenario04(self):
        """
        Scenario: Successfully building an images composite
            And I create a source from a "<zip>" of "<images_number>" images
            And I check that the composite is ready
            And I check that it has "<images_number>" components
        """
        print(self.test_scenario04.__doc__)
        headers = ["data", "output_dir"]
        examples = [
            ['data/images/fruits_hist.zip', './scenario42_04']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            source_create.i_create_source( \
                self, data=example["data"], output_dir=example["output_dir"])
            composite_create.i_check_create_composite(self)
            self.sources = world.source["object"].get("sources", [])
            composite_create.check_images_number_in_composite(
                self, example["data"])

    def test_scenario05(self):
        """
        Scenario: Successfully building an images composite from directory
            And I create a source from a "<directory>" in <output_dir>
            And I check that the composite is ready
            And I check that it has the expected components
        """
        print(self.test_scenario05.__doc__)
        headers = ["directory", "output_dir"]
        examples = [
            ['data/images/fruits_hist/', './scenario42_05']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            source_create.i_create_source( \
                self, data=example["directory"],
                output_dir=example["output_dir"])
            composite_create.i_check_create_composite(self)
            self.sources = world.source["object"].get("sources", [])
            composite_create.check_images_number_in_composite(
                self, example["directory"])

    def test_scenario06(self):
        """
        Scenario: Successfully building an annotated images composite
            And I create a source from an "<annotations_file>" and an "<images_file>"
            And I check that the composite is ready
            And I check that it has the expected annotation_fields
        """
        print(self.test_scenario06.__doc__)
        headers = ["images_file", "annotations_file", "output_dir"]
        examples = [
            ['data/images/fruits_hist.zip', 'data/images/annotations.json',
             './scenario42_06']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            composite_create.i_create_annotated_source( \
                self, images_file=example["images_file"],
                annotations_file=example["annotations_file"],
                output_dir=example["output_dir"])
            composite_create.i_check_create_composite(self)
            self.sources = world.source["object"].get("sources", [])
            composite_create.check_images_number_in_composite(
                self, example["images_file"])
            composite_create.check_annotation_fields(
                self, example["annotations_file"])

    def test_scenario07(self):
        """
        Scenario: Successfully building a <annotations_language> annotated images composite
            And I create a source from an "<annotations_dir>" and an "<images_dir>" using <language> and log in <output_dir>
            And I check that the composite is ready
            And I check that it has the expected annotation fields
        """
        print(self.test_scenario07.__doc__)
        headers = ["images_dir", "annotations_dir", "output_dir", "language"]
        examples = [
            ['data/images/fruits_hist', 'data/images/VOC_annotations',
             './scenario42_07_v', 'VOC'],
            ['data/images/YOLO_annotations', 'data/images/YOLO_annotations',
             './scenario42_07_y', 'YOLO']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            composite_create.i_create_lang_annotated_source(
                self, images_dir=example["images_dir"],
                annotations_dir=example["annotations_dir"],
                annotations_language=example["language"],
                output_dir=example["output_dir"])
            composite_create.i_check_create_composite(self)
            self.sources = world.source["object"].get("sources", [])
            composite_create.check_images_number_in_composite(
                self, example["images_dir"])
            composite_create.check_annotation_fields(
                self,
                os.path.join(example["output_dir"], "annotations.json"))

    def test_scenario08(self):
        """
        Scenario: Successfully extracting <type> features from images composite
            And I create a source from a "<zip>" and store output in "<output_dir>" directory
            And I check that the composite is ready
            And I update the source to extract <features>
            And I check that the image analysis reflects the extracted types
        """
        print(self.test_scenario08.__doc__)
        headers = ["zip", "output_dir", "features"]
        examples = [
            ['data/images/fruits_hist.zip', './scenario42_08',
             [["dimensions"], ["level_histogram"], ["dimensions", "HOG"],
              ["dimensions", "average_pixels", "level_histogram", "HOG"]]]]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            source_create.i_create_source( \
                self, data=example["zip"], output_dir=example["output_dir"])
            composite_create.i_check_create_composite(self)
            for extracted_features in example["features"]:
                composite_create.i_extract_features(self, extracted_features,
                    example["output_dir"])
                composite_create.i_check_create_composite(self)
                composite_create.i_check_extracted_features(self,
                    extracted_features)

    def test_scenario09(self):
        """
        Scenario: Successfully extracting <type2> features from images composite
            And I create a source from a "<zip>" and store output in "<output_dir>" directory
            And I check that the composite is ready
            And I update the source to extract <feature> <option>
            And I check that the image analysis reflects the extracted features
        """
        print(self.test_scenario09.__doc__)
        headers = ["zip", "output_dir", "feature", "option"]
        examples = [
            ['data/images/fruits_hist.zip', './scenario42_09',
             "ws-level", 2],
            ['data/images/fruits_hist.zip', './scenario42_09',
             "pretrained-cnn", "mobilenet"]]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            source_create.i_create_source( \
                self, data=example["zip"], output_dir=example["output_dir"])
            composite_create.i_check_create_composite(self)
            composite_create.i_extract_t2_features(self, example["feature"],
                example["option"], example["output_dir"])
            composite_create.i_check_create_composite(self)
            composite_create.i_check_extracted_t2_features(
                self, example["feature"], example["option"])



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
# Copyright 2017-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Tree level output for MySQL function

This module defines functions that generate MySQL SQL code to make local
predictions
"""

from bigml.tree_utils import INDENT
from bigml.generators.model import PYTHON_OPERATOR, missing_branch, \
    none_value
from bigml.predict_utils.common import mintree_split, get_predicate, get_node
from bigml.predict_utils.common import OPERATION_OFFSET, FIELD_OFFSET, \
    VALUE_OFFSET, MISSING_OFFSET
from bigml.generators.tree_common import filter_nodes

T_MISSING_OPERATOR = {
    "=": "ISNULL(",
    "!=": "NOT ISNULL("
}

# Map operator str to its corresponding mysql operator
MYSQL_OPERATOR = {
    "/=": "!="}

def value_to_print(value, optype):
    """String of code that represents a value according to its type

    """
    if value is None:
        return "NULL"
    if optype == 'numeric':
        return value
    return "'%s'" % value.replace("'", '\\\'')


def missing_check_code(tree, offsets, fields, objective_id,
                       field, alternate, cmv, attr=None):
    """Builds the code to predict when the field is missing

    """
    node = get_node(tree)
    condition = "ISNULL(`%s`)" % fields[field]['name']
    code = ("%s (%s)" %
             (alternate, condition))

    # used when printing the confidence metric
    if attr is not None:
        value = node[offsets[attr]]
    else:
        value = value_to_print( \
            node[offsets["output"]],
            fields[objective_id]['optype'])

    code += (", %s" % value)
    cmv.append(fields[field]['name'])

    return code

def missing_prefix_code(tree, fields, field, cmv):
    """Part of the condition that checks for missings when missing_splits
    has been used

    """

    predicate = get_predicate(tree)
    missing = predicate[MISSING_OFFSET]
    negation = "" if missing else "NOT "
    connection = "OR" if missing else "AND"
    if not missing:
        cmv.append(fields[field]['name'])
    return "(%sISNULL(`%s`) %s " % ( \
        negation, fields[field]['name'],
        connection)

def split_condition_code(tree, fields, field, alternate,
                         pre_condition):
    """Condition code for the split

    """

    predicate = get_predicate(tree)
    post_condition = ""
    optype = fields[field]['optype']
    value = value_to_print(predicate[VALUE_OFFSET], optype)
    operation = predicate[OPERATION_OFFSET]
    operator = ("" if predicate[VALUE_OFFSET] is None else
                MYSQL_OPERATOR.get(operation,
                                   PYTHON_OPERATOR.get(operation)))
    if predicate[VALUE_OFFSET] is None:
        value = ""
        pre_condition = (
            T_MISSING_OPERATOR[operation])
        post_condition = ")"

    condition = "%s`%s`%s%s%s" % ( \
        pre_condition,
        fields[predicate[FIELD_OFFSET]]['name'],
        operator,
        value,
        post_condition)
    return "%s (%s)" % (alternate, condition)

def plug_in_body(tree, offsets, fields, objective_id, depth=0, cmv=None,
                 ids_path=None, subtree=True, body="", attr=None):
    """Translate the model into a mysql function

    `depth` controls the size of indentation. As soon as a value is missing
    that node is returned without further evaluation.
    `attr` is used to decide the value returned by the function. When
    it's set to None, the prediction is returned. When set to the
    name of an attribute (e.g. 'confidence') this attribute is returned

    """
    if cmv is None:
        cmv = []

    if body:
        alternate = ",\n%sIF (" % (depth * INDENT)
    else:
        alternate = "IF ("
    post_missing_body = ""

    node = get_node(tree)
    children = filter_nodes([] if node[offsets["children#"]] == 0 \
        else node[offsets["children"]], offsets, ids=ids_path, subtree=subtree)

    if children:

        # field used in the split
        field = mintree_split(children)

        has_missing_branch = (missing_branch(children) or
                              none_value(children))
        # the missing is singled out as a special case only when there's
        # no missing branch in the children list
        if (not has_missing_branch and
                not fields[field]['name'] in cmv):
            body += missing_check_code(tree, offsets, fields, objective_id,
                                       field, alternate, cmv, attr)
            depth += 1
            alternate = ",\n%sIF (" % (depth * INDENT)
            post_missing_body += ")"

        for child in children:
            pre_condition = ""
            predicate = get_predicate(child)
            # code when missing splits has been used
            if has_missing_branch and predicate[VALUE_OFFSET] is not None:
                pre_condition = missing_prefix_code(child, fields, field, cmv)

            # complete split condition code
            body += split_condition_code( \
                child, fields, field, alternate, pre_condition)

            depth += 1
            alternate = ",\n%sIF (" % (depth * INDENT)
            body = plug_in_body(child, offsets, fields, objective_id,
                                depth, cmv=cmv[:],
                                ids_path=ids_path, subtree=subtree,
                                body=body, attr=attr)
        body += ", NULL))" + post_missing_body
        post_missing_body = ""
    else:
        if attr is None:
            value = value_to_print( \
                node[offsets["output"]], fields[objective_id]['optype'])
        else:
            try:
                value = node[offsets[attr]]
            except KeyError:
                value = "NULL"
        body += ", %s" % (value)

    return body



# -*- coding: utf-8 -*-
#
# Copyright 2020-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Resources management functions

"""


import sys

import bigml.api

from bigmler.utils import (dated, get_url, log_message, check_resource,
                           plural,
                           check_resource_error, log_created_resources)
from bigmler.reports import report

from bigmler.resourcesapi.common import set_basic_model_args, \
    update_json_args, configure_input_fields, update_sample_parameters_args,\
    wait_for_available_tasks, relative_input_fields, update_attributes
from bigmler.labels import label_model_args, get_all_labels
from bigmler.resourcesapi.common import SEED, EVALUATE_SAMPLE_RATE, \
    ALL_FIELDS_QS, BOOSTING_OPTIONS


def set_label_ensemble_args(args, labels, multi_label_data,
                            number_of_ensembles, fields):
    """Set of args needed to build an ensemble per label

    """

    if not args.model_fields_:
        args.model_fields_ = relative_input_fields(fields, args.model_fields_)
    if args.objective_field is None:
        args.objective_field = fields.objective_field
    try:
        objective_id = fields.field_id(args.objective_field)
    except ValueError as exc:
        sys.exit(exc)
    objective_field = fields.fields[objective_id]['name']
    ensemble_args_list = []

    for index in range(number_of_ensembles - 1, -1, -1):
        label = labels[index]
        all_labels = get_all_labels(multi_label_data)
        (new_name, label_field, single_label_fields) = label_model_args(
            args.name, label, all_labels, args.model_fields_,
            objective_field)
        ensemble_args = set_ensemble_args(args, name=new_name,
                                          objective_id=label_field,
                                          model_fields=single_label_fields,
                                          fields=fields)
        if multi_label_data is not None:
            ensemble_args.update(
                user_metadata={'multi_label_data': multi_label_data})

        ensemble_args_list.append(ensemble_args)
    return ensemble_args_list


def set_ensemble_args(args, name=None,
                      objective_id=None, model_fields=None, fields=None):
    """Return ensemble arguments dict

    """
    if name is None:
        name = args.name
    if objective_id is None:
        objective_id = args.objective_id_
    if model_fields is None:
        model_fields = args.model_fields_

    ensemble_args = set_basic_model_args(args, name)
    ensemble_args.update({
        "missing_splits": args.missing_splits,
        "ensemble_sample": {"seed": SEED if args.ensemble_sample_seed is None \
            else args.ensemble_sample_seed},
        "seed": SEED if args.seed is None else args.seed
    })
    if objective_id is not None and fields is not None:
        ensemble_args.update({"objective_field": objective_id})

    if args.boosting:
        boosting_args = {}
        for option in BOOSTING_OPTIONS:
            if hasattr(args, option) and getattr(args, option) is not None:
                boosting_args.update({option: getattr(args, option)})
        ensemble_args.update({"boosting": boosting_args})
    else:
        ensemble_args.update({"number_of_models": args.number_of_models})

    # If evaluate flag is on and no test_split flag is provided,
    # we choose a deterministic sampling with
    # args.sample_rate (80% by default) of the data to create the model

    if (args.evaluate and args.test_split == 0 and
            args.test_datasets is None and not args.dataset_off):
        ensemble_args.update({"seed": SEED})
        if args.sample_rate == 1:
            args.sample_rate = EVALUATE_SAMPLE_RATE

    if model_fields and fields is not None:
        input_fields = configure_input_fields(fields, model_fields)
        ensemble_args.update(input_fields=input_fields)

    if args.pruning and args.pruning != 'smart':
        ensemble_args.update(stat_pruning=(args.pruning == 'statistical'))
    if args.node_threshold > 0:
        ensemble_args.update(node_threshold=args.node_threshold)
    if args.balance:
        ensemble_args.update(balance_objective=True)
    if args.weight_field:
        try:
            weight_field = fields.field_id(args.weight_field)
        except ValueError as exc:
            sys.exit(exc)
        ensemble_args.update(weight_field=weight_field)
    if args.objective_weights:
        ensemble_args.update(objective_weights=args.objective_weights_json)
    if args.random_candidates:
        ensemble_args.update(random_candidates=args.random_candidates)

    update_attributes(ensemble_args, args.json_args.get('model'))

    ensemble_args = update_sample_parameters_args(ensemble_args, args)

    ensemble_args["ensemble_sample"].update( \
        {"rate": args.ensemble_sample_rate,
         "replacement": args.ensemble_sample_replacement})

    if 'ensemble' in args.json_args:
        update_json_args(ensemble_args, args.json_args.get('ensemble'), fields)

    return ensemble_args


def create_ensembles(datasets, ensemble_ids, ensemble_args, args,
                     number_of_ensembles=1,
                     api=None, path=None, session_file=None, log=None):
    """Create ensembles from input data

    """

    if api is None:
        api = bigml.api.BigML()
    ensembles = ensemble_ids[:]
    existing_ensembles = len(ensembles)
    model_ids = []
    ensemble_args_list = []
    if isinstance(ensemble_args, list):
        ensemble_args_list = ensemble_args
    if args.dataset_off and args.evaluate:
        args.test_dataset_ids = datasets[:]
    if not args.multi_label:
        datasets = datasets[existing_ensembles:]
    if number_of_ensembles > 0:
        message = dated("Creating %s.\n" %
                        plural("ensemble", number_of_ensembles))
        log_message(message, log_file=session_file,
                    console=args.verbosity)
        inprogress = []
        for i in range(0, number_of_ensembles):
            wait_for_available_tasks(inprogress, args.max_parallel_ensembles,
                                     api, "ensemble",
                                     wait_step=args.number_of_models)

            if ensemble_args_list:
                ensemble_args = ensemble_args_list[i]

            if args.dataset_off and args.evaluate:
                multi_dataset = args.test_dataset_ids[:]
                del multi_dataset[i + existing_ensembles]
                ensemble = api.create_ensemble(multi_dataset,
                                               ensemble_args,
                                               retries=None)
            else:
                ensemble = api.create_ensemble(datasets, ensemble_args,
                                               retries=None)
            ensemble_id = check_resource_error(ensemble,
                                               "Failed to create ensemble: ")
            log_message("%s\n" % ensemble_id, log_file=log)
            ensemble_ids.append(ensemble_id)
            inprogress.append(ensemble_id)
            ensembles.append(ensemble)
            log_created_resources("ensembles", path, ensemble_id,
                                  mode='a')
        models, model_ids = retrieve_ensembles_models(ensembles, api, path)
        if number_of_ensembles < 2 and args.verbosity:
            message = dated("Ensemble created: %s\n" %
                            get_url(ensemble))
            log_message(message, log_file=session_file,
                        console=args.verbosity)
            if args.reports:
                report(args.reports, path, ensemble)

    return ensembles, ensemble_ids, models, model_ids


def retrieve_ensembles_models(ensembles, api, path=None):
    """Retrieves the models associated to a list of ensembles

    """
    models = []
    model_ids = []
    for index, ensemble in enumerate(ensembles):
        if (isinstance(ensemble, str) or
                bigml.api.get_status(ensemble)['code'] != bigml.api.FINISHED):
            try:
                ensemble = check_resource(ensemble, api.get_ensemble,
                                          raise_on_error=True)
                ensembles[index] = ensemble
            except Exception as exception:
                sys.exit("Failed to get a finished ensemble: %s" %
                         str(exception))
        model_ids.extend(ensemble['object']['models'])
    if path is not None:
        for model_id in model_ids:
            log_created_resources("models", path, model_id, mode='a')
    models = model_ids[:]
    models[0] = check_resource(models[0], api.get_model,
                               query_string=ALL_FIELDS_QS,
                               raise_on_error=True)
    return models, model_ids


def get_ensemble(ensemble, api=None, verbosity=True, session_file=None):
    """Retrieves remote ensemble in its actual status

    """
    if api is None:
        api = bigml.api.BigML()
    if (isinstance(ensemble, str) or
            bigml.api.get_status(ensemble)['code'] != bigml.api.FINISHED):
        message = dated("Retrieving ensemble. %s\n" %
                        get_url(ensemble))
        log_message(message, log_file=session_file,
                    console=verbosity)
        ensemble = check_resource(ensemble, api.get_ensemble,
                                  raise_on_error=True)
        check_resource_error(ensemble, "Failed to get ensemble: ")
    return ensemble



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
# Copyright 2017-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Tree level output for node/js

This module defines functions that generate node/js code to make local
predictions
"""

from bigml.tree_utils import (
    INDENT, PYTHON_OPERATOR, MAX_ARGS_LENGTH, COMPOSED_FIELDS,
    NUMERIC_VALUE_FIELDS)
from bigml.predict_utils.common import mintree_split, get_predicate, get_node, \
    missing_branch
from bigml.predict_utils.common import OPERATION_OFFSET, FIELD_OFFSET, \
    VALUE_OFFSET, TERM_OFFSET, MISSING_OFFSET
from bigml.generators.tree_common import filter_nodes
from bigml.util import NUMERIC



def value_to_print(value, optype):
    """String of code that represents a value according to its type

    """
    if value is None:
        return "null"
    if optype in NUMERIC_VALUE_FIELDS: # value is numeric for these fields
        return value
    return "\"%s\"" % value.replace('"', '\\"')


def missing_check_code(tree, offsets, fields, objective_id,
                       field, depth, prefix, cmv, metric):
    """Builds the code to predict when the field is missing

    """
    node = get_node(tree)
    code = "%sif (%s%s == null) {\n" % \
        (INDENT * depth,
         prefix,
         fields[field]['camelCase'])
    value = value_to_print(node[offsets["output"]],
                           fields[objective_id]['optype'])
    code += "%sreturn {prediction: %s," \
        " %s: %s}}\n" % \
        (INDENT * (depth + 1), value, metric, node[offsets["confidence"]])
    cmv.append(fields[field]['camelCase'])
    return code


def missing_prefix_code(tree, fields, field, prefix, cmv):
    """Part of the condition that checks for missings when missing_splits
    has been used

    """

    predicate = get_predicate(tree)
    missing = predicate[MISSING_OFFSET]
    operator = "==" if missing else "!="
    connection = "||" if missing else "&&"
    if not missing:
        cmv.append(fields[field]['camelCase'])
    return "%s%s %s null %s " % (prefix,
                                  fields[field]['camelCase'],
                                  operator,
                                  connection)


def split_condition_code(tree, fields, field, depth, prefix,
                         pre_condition, term_analysis_fields,
                         item_analysis_fields, cmv):
    """Condition code for the split

    """
    optype = fields[field]['optype']
    predicate = get_predicate(tree)
    operator = PYTHON_OPERATOR[predicate[OPERATION_OFFSET]]
    value = value_to_print(predicate[VALUE_OFFSET], optype)

    if optype in ['text', 'items']:
        if optype == 'text':
            term_analysis_fields.append((field,
                                         predicate[TERM_OFFSET]))
            matching_function = "termMatches"
        else:
            item_analysis_fields.append((field,
                                         predicate[TERM_OFFSET]))
            matching_function = "itemMatches"

        return "%sif (%s%s(%s%s, %s, %s) %s %s) {\n" % \
            (INDENT * depth,
             pre_condition,
             matching_function,
             prefix,
             fields[field]['camelCase'],
             value_to_print(fields[field]['camelCase'],
                 'categorical'),
             value_to_print(predicate[TERM_OFFSET], 'categorical'),
             operator,
             value)
    if value is None:
        cmv.append(fields[field]['camelCase'])
    return "%sif (%s%s%s %s %s) {\n" % \
        (INDENT * depth, pre_condition,
         prefix,
         fields[field]['camelCase'],
         operator,
         value)


def plug_in_body(tree, offsets, fields, objective_id,
                 depth=1, cmv=None, ids_path=None, subtree=True):
    """Translate the model into a set of "if" javascript statements.

    `depth` controls the size of indentation. As soon as a value is missing
    to evaluate a predicate the output at that node is returned without
    further evaluation.

    """
    if cmv is None:
        cmv = []
    body = ""
    term_analysis_fields = []
    item_analysis_fields = []
    prefix = ""
    field_obj = fields[objective_id]
    metric = "error" if field_obj["optype"] == NUMERIC \
        else "confidence"


    if len(fields) > MAX_ARGS_LENGTH:
        prefix = "data."

    node = get_node(tree)
    children = filter_nodes([] if node[offsets["children#"]] == 0 else \
                            node[offsets["children"]], offsets, ids=ids_path,
                            subtree=subtree)

    if children:

        # field used in the split
        field = mintree_split(children)

        has_missing_branch = missing_branch(children)
        # the missing is singled out as a special case only when there's
        # no missing branch in the children list
        one_branch = not has_missing_branch or \
            fields[field]['optype'] in COMPOSED_FIELDS
        if (one_branch and
                not fields[field]['camelCase'] in cmv):
            body += missing_check_code(tree, offsets, fields, objective_id,
                                       field, depth, prefix, cmv, metric)

        for child in children:

            predicate = get_predicate(child)
            field = predicate[FIELD_OFFSET]

            pre_condition = ""
            # code when missing_splits has been used
            if has_missing_branch and predicate[VALUE_OFFSET] is not None:
                pre_condition = missing_prefix_code(child, fields,
                                                    field, prefix, cmv)

            # complete split condition code
            body += split_condition_code( \
                child, fields,
                field, depth, prefix, pre_condition,
                term_analysis_fields, item_analysis_fields, cmv)

            # value to be determined in next node
            next_level = plug_in_body(child, offsets, fields, objective_id,
                                      depth + 1, cmv=cmv[:],
                                      ids_path=ids_path, subtree=subtree)
            body += next_level[0]
            body += "%s}\n" % (INDENT * depth)
            term_analysis_fields.extend(next_level[1])
            item_analysis_fields.extend(next_level[2])

    else:
        value = value_to_print(node[offsets["output"]],
                               fields[objective_id]['optype'])
        body = "%sreturn {prediction: %s, %s: %s};\n" % ( \
            INDENT * depth,
            value,
            metric,
            node[offsets["confidence"]])
    return body, term_analysis_fields, item_analysis_fields



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2017-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os
import shutil

from bigml.api import check_resource

from bigmler.tests.common_steps import shell_execute, check_rows_equal
from bigmler.tests.world import world, res_filename, ok_


def i_create_all_ts_resources_with_no_headers(
    step, data=None, output=None):
    """Step: I create BigML time series resources uploading train <data>
    file with no headers and log forecasts in <output>
    """
    ok_(data is not None and output is not None)
    command = ("bigmler time-series --train " + res_filename(data) +
               " --store --output " + output +
               " --horizon 10 --no-train-header --no-test-header")
    shell_execute(command, output, options='--prediction-header')


def i_create_all_ts_resources(step, data=None, test=None, output=None):
    """Step: I create BigML time series resources uploading train <data>
    file to test <test> and log forecasts in <output>
    """
    ok_(data is not None and test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler time-series --train " + res_filename(data) +
               " --test " + test +
               " --store --output " + output)
    shell_execute(command, output, test=test)


def i_create_all_ts_resources_to_evaluate(step, data=None, output=None):
    """Step: I create BigML time series resources uploading train <data> file
    to evaluate and log evaluation in <output>
    """
    ok_(data is not None and output is not None)
    command = ("bigmler time-series --train " + res_filename(data) +
               " --evaluate" +
               " --store --output " + output)
    shell_execute(command, output)


def i_check_create_time_series(step):
    """Step: I check that the time series has been created"""
    ts_file = os.path.join(world.directory, "time_series")
    message = None
    #pylint: disable=locally-disabled,import-outside-toplevel
    try:
        with open(ts_file) as handler:
            ts_ = check_resource(handler.readline().strip(),
                                 world.api.get_time_series)
            world.time_series_set.append(ts_['resource'])
            world.time_series = ts_
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_forecasts(step, check_file):
    """Step: the local forecasts file is like <check_file>"""
    check_file_path = res_filename(check_file)
    forecasts_file_path = "%s_%s.csv" % \
        (world.output, world.time_series["object"]["objective_field"])
    message = None
    #pylint: disable=locally-disabled,import-outside-toplevel
    try:
        message = check_rows_equal(forecasts_file_path,
            check_file_path)
    except Exception:
        import traceback
        shutil.copyfile(forecasts_file_path, "%s.new" % check_file_path)
        message = traceback.format_exc()
    ok_(message is None, msg=message)



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2015-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing Multi-label predictions

"""
from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)

import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.ml_tst_prediction_steps as ml_pred


def setup_module():
    """Setup for the module

    """
    common_setup_module()
    test = TestMLM()
    test.bigml = {"method": "setup_scenario1"}
    test.setup_scenario1()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()

class TestMLM:
    """Testing multilabel commands"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def setup_scenario1(self):
        """
        Scenario: Successfully building multi-label test predictions from start:
            Given I create BigML multi-label resources tagged as "<tag>" with "<label_separator>" label separator and <number_of_labels> labels uploading train "<data>" file with "<training_separator>" field separator and "<ml_fields>" as multi-label fields using model_fields "<model_fields>" and objective "<objective>" to test "<test>" and log predictions in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the models have been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.setup_scenario1.__doc__)
        headers = ["tag", "label_separator", "number_of_labels", "data",
                   "training_separator", "ml_fields", "model_fields",
                   "objective", "test", "output", "predictions_file"]
        examples = [
            ['my_multilabelm_1', ':', '7', 'data/multilabel_multi.csv', ',',
             'type,class', '-type,-type - W,-type - A,-type - C,-type - S,' +
             '-type - R,-type - T,-type - P', 'class',
             'data/test_multilabel.csv',
             'scenario_mlm_1/predictions.csv',
             'check_files/predictions_ml.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            ml_pred.i_create_all_mlm_resources(self, tag=example["tag"],
                label_separator=example["label_separator"],
                number_of_labels=example["number_of_labels"],
                data=example["data"],
                training_separator=example["training_separator"],
                ml_fields=example["ml_fields"],
                model_fields=example["model_fields"],
                objective=example["objective"],
                test=example["test"],
                output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_pred.i_check_create_models(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario2(self):
        """
        Scenario: Successfully building test predictions from source
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML multi-label resources using source with objective "<objective>" and model fields "<model_fields>" to test "<test>" and log predictions in "<output>"
            And I check that the dataset has been created
            And I check that the models have been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario2.__doc__)
        headers = ["scenario", "kwargs", "objective", "model_fields",
                   "test", "output", "predictions_file"]
        examples = [
            ['scenario_mlm_1', '{"tag": "my_multilabelm_1",' +
             ' "data": "data/multilabel_multi.csv", "label_separator": ":",' +
             ' "number_of_labels": 7, "training_separator": ",",' +
             ' "output": "scenario_mlm_1/predictions.csv",' +
             ' "test": "data/test_multilabel.csv",' +
             ' "ml_fields": "type,class", "model_fields": "-type,' +
             '-type - W,-type - A,-type - C,-type - S,-type - R,-type - T,' +
             '-type - P", "objective": "class"}', 'class',
             '-type,-type - W,-type - A,-type - C,-type - S,-type - R,' +
             '-type - T,-type - P', 'data/test_multilabel.csv',
             'scenario_mlm_2/predictions.csv',
             'check_files/predictions_ml_comma.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_source_with_objective(
                self, multi_label='multi-label ',
                objective=example["objective"],
                model_fields=example["model_fields"],
                test=example["test"],
                output=example["output"])
            test_pred.i_check_create_dataset(self)
            test_pred.i_check_create_models(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario3(self):
        """
        Scenario: Successfully building test predictions from dataset
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML multi-label resources using dataset with objective "<objective>" and model fields "<model_fields>" to test "<test>" and log predictions in "<output>"
            And I check that the models have been created
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario3.__doc__)
        headers = ["scenario", "kwargs", "objective", "model_fields",
                   "test", "output", "predictions_file"]
        examples = [
            ['scenario_mlm_1', '{"tag": "my_multilabelm_1",' +
             ' "data": "data/multilabel_multi.csv", "label_separator": ":",' +
             ' "number_of_labels": 7, "training_separator": ",",' +
             ' "output": "scenario_mlm_1/predictions.csv",' +
             ' "test": "data/test_multilabel.csv",' +
             ' "ml_fields": "type,class", "model_fields": "-type,' +
             '-type - W,-type - A,-type - C,-type - S,-type - R,' +
             '-type - T,-type - P", "objective": "class"}',
             'class', '-type,-type - W,-type - A,-type - C,' +
             '-type - S,-type - R,-type - T,-type - P',
             'data/test_multilabel.csv', 'scenario_mlm_3/predictions.csv',
             'check_files/predictions_ml_comma.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_dataset_with_objective(
                self, multi_label='multi-label ',
                objective=example["objective"],
                model_fields=example["model_fields"],
                test=example["test"],
                output=example["output"])
            test_pred.i_check_create_models(self)
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario4(self):
        """
        Scenario: Successfully building test predictions from models file
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML multi-label resources using models in file "<models_file>" with objective "<objective>" to test "<test>" and log predictions in "<output>"
            And I check that the predictions are ready
            Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario4.__doc__)
        headers = ["scenario", "kwargs", "models_file", "objective",
                   "test", "output", "predictions_file"]
        examples = [
            ['scenario_mlm_1', '{"tag": "my_multilabelm_1",' +
             ' "data": "data/multilabel_multi.csv", "label_separator": ":",' +
             ' "number_of_labels": 7, "training_separator": ",",' +
             ' "output": "scenario_mlm_1/predictions.csv",' +
             ' "test": "data/test_multilabel.csv",' +
             ' "ml_fields": "type,class", "model_fields": "-type,' +
             '-type - W,-type - A,-type - C,-type - S,-type - R,' +
             '-type - T,-type - P", "objective": "class"}',
             'scenario_mlm_1/models', 'class', 'data/test_multilabel.csv',
             'scenario_mlm_4/predictions.csv',
             'check_files/predictions_ml_comma.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_pred.i_create_resources_from_models_file_with_objective(
                self, multi_label='multi-label ',
                models_file=example["models_file"],
                objective=example["objective"],
                test=example["test"], output=example["output"])
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario5(self):
        """
            Scenario: Successfully building test predictions from models retrieved by tag
                Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
                And I create BigML multi-label resources with labels "<labels>" and objective "<objective>" using models tagged as "<tag>" to test "<test>" and log predictions in "<output>"
                And I check that the predictions are ready
                Then the local prediction file is like "<predictions_file>"
        """
        print(self.test_scenario5.__doc__)
        headers = ["scenario", "kwargs", "labels", "objective", "tag",
                   "test", "output", "predictions_file"]
        examples = [
            ['scenario_mlm_1', '{"tag": "my_multilabelm_1",' +
             ' "data": "data/multilabel_multi.csv",' +
             ' "label_separator": ":", "number_of_labels": 7,' +
             ' "training_separator": ",",' +
             ' "output": "scenario_mlm_1/predictions.csv",' +
             ' "test": "data/test_multilabel.csv",' +
             ' "ml_fields": "type,class", "model_fields": "-type,' +
             '-type - W,-type - A,-type - C,-type - S,-type - R,' +
             '-type - T,-type - P", "objective": "class"}', 'Adult,Student',
             'class', 'my_multilabelm_1', 'data/test_multilabel.csv',
             'scenario_mlm_5/predictions.csv',
             'check_files/predictions_ml_labels.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            ml_pred.i_predict_ml_from_model_tag_with_labels_with_objective(
                self, labels=example["labels"], objective=example["objective"],
                tag=example["tag"], test=example["test"],
                output=example["output"])
            test_pred.i_check_create_predictions(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario6(self):
        """
        Scenario: Successfully extending the multi-label source file:
            Given I create BigML a multi-label source with "<label_separator>" label separator and <number_of_labels> labels from train "<data>" file with "<training_separator>" field separator and "<ml_fields>" as multi-label fields and objective "<objective>" and output in "<output_dir>"
            And I check that the source has been created
            Then I check the extended file "<local_file>" has been created
            And the headers of the local extended file are "<headers>"
            And the first row of the local extended file is "<first_row>"
        """
        print(self.test_scenario6.__doc__)
        headers = ["label_separator", "number_of_labels", "data",
                   "training_separator", "ml_fields", "objective",
                   "output_dir", "local_file", "headers", "first_row"]
        examples = [
            [':', '7', 'data/multilabel_multi.csv', ',', 'type,class', 'class',
             'scenario_mlm_6', 'scenario_mlm_6/extended_multilabel_multi.csv',
             'color,year,price,first_name,last_name,sex,class,type,' +
             'class - Adult,class - Child,class - Pensioner,class - Retired,' +
             'class - Student,class - Teenager,class - Worker,type - A,' +
             'type - C,type - P,type - R,type - S,type - T,type - W',
             'Blue,1992,"1208,6988040134",John,Higgins,Male,' +
             'Worker:Adult,W:A:C:S:T:R:P,1,0,0,0,0,0,1,1,1,1,1,1,1,1'],
            [':', '7', 'data/multilabel_multi2.csv', ',',
             'Colors,Movies,Hobbies', 'Hobbies', 'scenario_mlm_7',
             'scenario_mlm_7/extended_multilabel_multi2.csv',
             'Registration Date,Age Range,Gender,Height,Weight,Points,' +
             'Colors,Movies,Hobbies,Colors - Black,Colors - Blue,' +
             'Colors - Green,Colors - Grey,Colors - Orange,' +
             'Colors - Pink,Colors - Purple,Colors - Red,Colors - White,' +
             'Colors - Yellow,Movies - Action,Movies - Adventure,' +
             'Movies - Comedy,Movies - Crime,Movies - Erotica,' +
             'Movies - Fantasy,Movies - Horror,Movies - Mystery,' +
             'Movies - Philosophical,Movies - Political,Movies - Romance,' +
             'Movies - Satire,Movies - Thriller,Hobbies - Barbacue,' +
             'Hobbies - Books,Hobbies - Chat,Hobbies - Cooking,' +
             'Hobbies - Dance,Hobbies - Disco,Hobbies - Dolls,' +
             'Hobbies - Family,Hobbies - Films,Hobbies - Fishing,' +
             'Hobbies - Friends,Hobbies - Jogging,Hobbies - Music,' +
             'Hobbies - Soccer,Hobbies - Toys,Hobbies - Travel,' +
             'Hobbies - Videogames,Hobbies - Walking',
             '2011-02-06,19-30,Female,140,47,11,White:Red,Comedy:Romance,' +
             'Friends:Music,0,0,0,0,0,0,0,1,1,0,0,0,1,0,0,0,0,0,0,0,1,0,' +
             '0,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0']
]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            ml_pred.i_create_ml_source(
                self, label_separator=example["label_separator"],
                number_of_labels=example["number_of_labels"],
                data=example["data"],
                training_separator=example["training_separator"],
                multi_label_fields=example["ml_fields"],
                objective=example["objective"],
                output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            ml_pred.i_check_local_file(self, path=example["local_file"])
            ml_pred.i_check_headers_file(
                self, headers=example["headers"])
            ml_pred.i_check_first_row_file(
                self, first_row=example["first_row"])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2022-2023 BigML, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


"""Defines a Reader object to return the files in a folder """

import os


class FolderReader():
    """Adapter to read files in a folder

    """
    def __init__(self, folder, filter_fn=None, header=None):
        """Constructor method for the reader

        """
        self.folder = folder
        self.filter_fn = lambda x: True if filter_fn is None else filter_fn
        self.header = header
        self.reader = [] if header is None else [[header]]

    def open_reader(self):
        """exploring the folder

        """
        self.reader.extend([[filename] for filename in
            os.listdir(self.folder) if self.filter_fn(filename)])
        for filename in self.reader:
            yield filename



# -*- coding: utf-8 -*-
#
# Copyright 2020-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Resources management functions

"""


import sys

import bigml.api

from bigmler.utils import (dated, get_url, log_message, check_resource,
                           check_resource_error, log_created_resources)
from bigmler.reports import report

from bigmler.resourcesapi.common import set_basic_batch_args, map_fields, \
    update_json_args

from bigmler.resourcesapi.common import FULL_FORMAT, NORMAL_FORMAT, \
    THRESHOLD_CODE



def set_batch_prediction_args(args, fields=None,
                              dataset_fields=None):
    """Return batch prediction args dict

    """
    batch_prediction_args = set_basic_batch_args(args, args.name)

    if hasattr(args, 'method') and args.method:
        batch_prediction_args.update({"combiner": args.method})
        if args.method == THRESHOLD_CODE:
            threshold = {}
            if hasattr(args, 'threshold') and args.threshold is not None:
                threshold.update(k=args.threshold)
            if hasattr(args, 'threshold_class') \
                    and args.threshold_class is not None:
                threshold.update({"class": args.threshold_class})
            batch_prediction_args.update(threshold=threshold)
    if args.fields_map_ and fields is not None:
        if dataset_fields is None:
            dataset_fields = fields
        batch_prediction_args.update({
            "fields_map": map_fields(args.fields_map_,
                                     fields, dataset_fields)})

    if args.prediction_info in [NORMAL_FORMAT, FULL_FORMAT]:
        if (hasattr(args, 'boosting') and args.boosting) or \
                (hasattr(args, 'probability') and args.probability):
            batch_prediction_args.update(probability=True)
        else:
            batch_prediction_args.update(confidence=True)

    if args.prediction_info == FULL_FORMAT:
        batch_prediction_args.update(all_fields=True)
        if hasattr(args, 'prediction_name') and args.prediction_name:
            batch_prediction_args.update(prediction_name=args.prediction_name)
    if args.prediction_fields:
        batch_prediction_args.update(all_fields=False)
        prediction_fields = []
        for field in args.prediction_fields.split(args.args_separator):
            field = field.strip()
            if not field in dataset_fields.fields:
                try:
                    field = dataset_fields.field_id(field)
                except ValueError as exc:
                    sys.exit(exc)
            prediction_fields.append(field)
        batch_prediction_args.update(output_fields=prediction_fields)
    if hasattr(args, 'missing_strategy') and args.missing_strategy:
        batch_prediction_args.update(missing_strategy=args.missing_strategy)
    if hasattr(args, "operating_point_") and args.operating_point_:
        batch_prediction_args.update(operating_point=args.operating_point_)
        if args.operating_point_.get("kind") == "probability":
            batch_prediction_args.update({"probability": True,
                                          "confidence": False})

    if 'batch_prediction' in args.json_args:
        update_json_args(
            batch_prediction_args,
            args.json_args.get('batch_prediction'),
            fields)
    return batch_prediction_args


def create_batch_prediction(model_or_ensemble, test_dataset,
                            batch_prediction_args, args,
                            api=None, session_file=None,
                            path=None, log=None):
    """Creates remote batch_prediction

    """
    if api is None:
        api = bigml.api.BigML()

    message = dated("Creating batch prediction.\n")
    log_message(message, log_file=session_file, console=args.verbosity)
    batch_prediction = api.create_batch_prediction(model_or_ensemble,
                                                   test_dataset,
                                                   batch_prediction_args,
                                                   retries=None)
    log_created_resources("batch_prediction", path,
                          bigml.api.get_batch_prediction_id(batch_prediction),
                          mode='a')
    batch_prediction_id = check_resource_error(
        batch_prediction, "Failed to create batch prediction: ")
    try:
        batch_prediction = check_resource(batch_prediction,
                                          api.get_batch_prediction,
                                          raise_on_error=True)
    except Exception as exception:
        sys.exit("Failed to get a finished batch prediction: %s"
                 % str(exception))
    message = dated("Batch prediction created: %s\n"
                    % get_url(batch_prediction))
    log_message(message, log_file=session_file, console=args.verbosity)
    log_message("%s\n" % batch_prediction_id, log_file=log)
    if args.reports:
        report(args.reports, path, batch_prediction)
    return batch_prediction



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2014-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.



import os

from bigml.api import check_resource, BigML

from bigmler.tests.common_steps import base_shell_execute
from bigmler.tests.world import world, res_filename, ok_, eq_


def i_create_source_with_project(
    step, data=None, project=None, output_dir=None):
    """Step: I create a BigML source uploading train <data> file and associate
    it to a new project named <project> storing results in <output_dir>
    """
    ok_(data is not None)
    #Check if the project already exists
    previous_projects = world.api.list_projects('name=%s' % project)
    while previous_projects['meta']['total_count'] > 0:
        print("the project %s already exists, trying with:" % project)
        project += " " + project
        print(project)
        previous_projects = world.api.list_projects('name=%s' % project)
    command = ("bigmler --train " + res_filename(data) +
               " --no-model --no-dataset --store --output-dir " +
               output_dir +
               " --project=\"" + project + "\"")
    base_shell_execute(command, output_dir)


def i_create_project_in_org(
    step, name=None, output_dir=None, organization=None):
    """Step: I create a BigML project in an organization named <name> storing
    results in <output>
    """
    ok_(name is not None and organization is not None)
    command = ("bigmler project --name \"" + name +
               "\" --organization " + organization +
               " --output-dir " + output_dir)
    base_shell_execute(command, output_dir)


def i_create_source_with_org_project(step, data=None, output_dir=None):
    """Step: I create a BigML source uploading train <data> file and associate
    it to the last created project id storing results in <output_dir>
    """
    ok_(data is not None)
    command = ("bigmler --train " + res_filename(data) +
               " --no-model --no-dataset --store --output-dir " +
               output_dir +
               " --org-project " + world.project["resource"])
    base_shell_execute(command, output_dir)


def i_create_source_with_project_id(step, data=None, output_dir=None):
    """Step: I create a BigML source uploading train <data> file and associate
    it to the last created project id storing results in <output_dir>
    """
    ok_(data is not None)
    command = ("bigmler --train " + res_filename(data) +
               " --no-model --no-dataset --store --output-dir " +
               output_dir +
               " --project-id " + world.project['resource'])
    base_shell_execute(command, output_dir)


def check_source_in_project(step):
    """Step: the source is associated to the project"""
    eq_(world.project['resource'], world.source['object']['project'])


def i_check_create_project(step, organization=False):
    """Step: I check that the project has been created"""
    project_file = os.path.join(world.directory, "project")
    message = None
    try:
        with open(project_file) as handler:
            project = check_resource(handler.readline().strip(),
                                     world.api.get_project)
            world.projects.append(project['resource'])
            world.project = project
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)
    if organization:
        world.api = BigML(world.username, world.api_key, debug=world.debug,
                          project=world.project["resource"])
        print(world.api.connection_info())


def check_source_in_no_project(step):
    """Step: the source has no project association"""
    ok_(world.source['object']['project'] is None)


def i_create_project(step, project=None, output_dir=None):
    """Step: I create a BigML project <project> and log results
    in <output_dir>
    """
    ok_(project is not None)
    command = ("bigmler project --name \"" + project +
               "\" --store --output-dir " +
               output_dir)
    base_shell_execute(command, output_dir)


def i_update_project(step, params=None, values=None):
    """Step: I update the project params <params> to <values>"""
    ok_(params is not None and values is not None)
    command = ("bigmler project --project-id " +
               world.project['resource'] +
               " --store --output-dir " +
               world.directory)
    for index, param in enumerate(params):
        value = values[index]
        command += " --%s %s" % (param, value)
    base_shell_execute(command, world.directory)


def check_params_values(step, params=None, values=None):
    """Step: the project params <params> are <values>"""
    for index, param in enumerate(params):
        value = values[index]
        eq_(world.project['object'][param], value)


def i_check_update_project(step):
    """Step: I check that the project has been updated"""
    project = check_resource(world.project['resource'],
                             world.api.get_project)
    world.project = project



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2015-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing evaluations' creation

"""
from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)


import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.basic_logistic_r_steps as lr_pred
import bigmler.tests.evaluation_steps as evaluation
import bigmler.tests.basic_time_series_steps as ts_pred
import bigmler.tests.basic_deepnet_steps as dn_pred


def setup_module():
    """Setup for the module

    """
    common_setup_module()
    test = TestEvaluation()
    test.bigml = {"method": "setup_scenario1"}
    test.setup_scenario1()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()


class TestEvaluation:
    """Testing evaluation commands"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def setup_scenario1(self):
        """
            Scenario: Successfully building evaluations from start:
                Given I create BigML resources uploading train "<data>" file to evaluate and log evaluation in "<output>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the model has been created
                And I check that the evaluation has been created
                Then the evaluation file is like "<json_evaluation_file>"|
        """
        print(self.setup_scenario1.__doc__)
        headers = ["data", "output", "json_evaluation_file"]
        examples = [
            ['data/iris.csv', 'scenario_e1/evaluation',
             'check_files/evaluation_iris.json']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_all_resources_to_evaluate(
                self, data=example["data"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_model(self)
            test_pred.i_check_create_evaluation(self)
            evaluation.then_the_evaluation_file_is_like(
                self, example["json_evaluation_file"])

    def test_scenario2(self):
        """
            Scenario: Successfully building evaluations from source
                Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
                And I create BigML resources using source to evaluate and log evaluation in "<output>"
                And I check that the dataset has been created
                And I check that the model has been created
                And I check that the evaluation has been created
                Then the evaluation file is like "<json_evaluation_file>"
        """
        print(self.test_scenario2.__doc__)
        headers = ["scenario", "kwargs", "output", "json_evaluation_file"]
        examples = [
            ['scenario_e1', '{"data": "data/iris.csv",' +
             ' "output": "scenario_e1/predictions.csv"}',
             'scenario_e2/evaluation', 'check_files/evaluation_iris.json']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            evaluation.given_i_create_bigml_resources_using_source_to_evaluate(
                self, output=example["output"])
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_model(self)
            test_pred.i_check_create_evaluation(self)
            evaluation.then_the_evaluation_file_is_like(
                self, example["json_evaluation_file"])

    def test_scenario3(self):
        """
            Scenario: Successfully building evaluations from dataset
                Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
                And I create BigML resources using dataset to evaluate and log evaluation in "<output>"
                And I check that the model has been created
                And I check that the evaluation has been created
                Then the evaluation file is like "<json_evaluation_file>"
        """
        print(self.test_scenario3.__doc__)
        headers = ["scenario", "kwargs", "output", "json_evaluation_file"]
        examples = [
            ['scenario_e1', '{"data": "data/iris.csv",' +
             ' "output": "scenario_e1/predictions.csv"}',
             'scenario_e3/evaluation', 'check_files/evaluation_iris.json']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            evaluation.given_i_create_bigml_resources_using_dataset_to_evaluate(
                self, output=example["output"])
            test_pred.i_check_create_model(self)
            test_pred.i_check_create_evaluation(self)
            evaluation.then_the_evaluation_file_is_like(
                self, example["json_evaluation_file"])

    def test_scenario4(self):
        """
            Scenario: Successfully building evaluation from model and test file
                Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
                And I create BigML resources using test file "<test>" to evaluate a model and log evaluation in "<output>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the evaluation has been created
                Then the evaluation file is like "<json_evaluation_file>"
        """
        print(self.test_scenario4.__doc__)
        headers = ["scenario", "kwargs", "data", "output", "json_evaluation_file"]
        examples = [
            ['scenario_e1', '{"data": "data/iris.csv",' +
             ' "output": "scenario_e1/predictions.csv"}',
             'data/iris.csv', 'scenario_e4/evaluation',
             'check_files/evaluation_iris2.json']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            evaluation.i_create_all_resources_to_evaluate_with_model(
                self, data=example["data"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_evaluation(self)
            evaluation.then_the_evaluation_file_is_like(
                self, example["json_evaluation_file"])

    def test_scenario5(self):
        """
            Scenario: Successfully building evaluation from model and test file with data map
                Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
                And I create BigML resources using test file "<test>" and a fields map "<fields_map>" to evaluate a model and log evaluation in "<output>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the evaluation has been created
                Then the evaluation file is like "<json_evaluation_file>"
        """
        print(self.test_scenario5.__doc__)
        headers = ["scenario", "kwargs", "test", "fields_map", "output",
                   "json_evaluation_file"]
        examples = [
            ['scenario_e1', '{"data": "data/iris.csv",' +
             ' "output": "scenario_e1/predictions.csv"}',
             'data/iris_permuted.csv', 'data/fields_map.csv',
             'scenario_e7/evaluation', 'check_files/evaluation_iris2.json']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            evaluation.i_create_all_resources_to_evaluate_with_model_and_map(
                self, data=example["test"], fields_map=example["fields_map"],
                output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_evaluation(self)
            evaluation.then_the_evaluation_file_is_like(
                self, example["json_evaluation_file"])

    def test_scenario6(self):
        """
            Scenario: Successfully building evaluations from start and test-split:
                Given I create BigML resources uploading train "<data>" file to evaluate with test-split <split> and log evaluation in "<output>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the train dataset has been created
                And I check that the test dataset has been created
                And I check that the model has been created
                And I check that the evaluation has been created
                Then the evaluation key "<key>" value for the model is greater than <value>
        """
        print(self.test_scenario6.__doc__)
        headers = ["data", "output", "split", "key", "value"]
        examples = [
            ['data/iris.csv', 'scenario_e6/evaluation', '0.2', 'average_phi', '0.85']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            evaluation.i_create_with_split_to_evaluate(
                self, data=example["data"], split=example["split"],
                output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_dataset(self, suffix='train ')
            test_pred.i_check_create_dataset(self, suffix='test ')
            test_pred.i_check_create_model(self)
            test_pred.i_check_create_evaluation(self)
            evaluation.i_check_evaluation_key(
                self, key=example["key"], value=example["value"])

    def test_scenario7(self):
        """
            Scenario: Successfully building ensemble evaluations from start and test-split:
                Given I create BigML resources uploading train "<data>" file to evaluate an ensemble of <number_of_models> models with test-split <split> and log evaluation in "<output>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the train dataset has been created
                And I check that the test dataset has been created
                And I check that the ensemble has been created
                And I check that the evaluation has been created
                Then the evaluation key "<key>" value for the model is greater than <value>
                And I evaluate the ensemble in directory "<directory>" with the dataset in directory "<directory>" and log evaluation in "<output2>"
                And I check that the evaluation has been created
                Then the evaluation key "<key>" value for the model is greater than <value>
        """
        print(self.test_scenario7.__doc__)
        headers = ["data", "output", "split", "number_of_models", "key",
                   "value", "directory", "output2"]
        examples = [
            ['data/iris.csv', 'scenario_e8/evaluation', '0.2', '5',
             'average_phi', '0.94', 'scenario_e8', 'scenario_e9/evaluation']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            evaluation.i_create_with_split_to_evaluate_ensemble(
                self, data=example["data"],
                number_of_models=example["number_of_models"],
                split=example["split"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_dataset(self, suffix='train ')
            test_pred.i_check_create_dataset(self, suffix='test ')
            test_pred.i_check_create_ensemble(self)
            test_pred.i_check_create_evaluation(self)
            evaluation.i_check_evaluation_key(
                self, key=example["key"], value=example["value"])
            evaluation.i_evaluate_ensemble_with_dataset(
                self, ensemble_dir=example["directory"],
                dataset_dir=example["directory"],
                output=example["output"])
            test_pred.i_check_create_evaluation(self)
            evaluation.i_check_evaluation_key(
                self, key=example["key"], value=example["value"])

    def test_scenario8(self):
        """
            Scenario: Successfully building evaluations for logistic regression from start:
                Given I create BigML logistic regression resources uploading train "<data>" file to evaluate and log evaluation in "<output>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the logistic regression has been created
                And I check that the evaluation has been created
                Then the evaluation file is like "<json_evaluation_file>"
        """
        print(self.test_scenario8.__doc__)
        headers = ["data", "output", "json_evaluation_file"]
        examples = [
            ['data/iris.csv', 'scenario_e8/evaluation',
             'check_files/evaluation_iris_lr.json']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            lr_pred.i_create_all_lr_resources_to_evaluate(
                self, data=example["data"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            lr_pred.i_check_create_lr_model(self)
            test_pred.i_check_create_evaluation(self)
            evaluation.then_the_evaluation_file_is_like(
                self, example["json_evaluation_file"])

    def test_scenario9(self):
        """
            Scenario: Successfully building ensemble evaluations from start and test-split:
                Given I create BigML resources uploading train "<data>" file to evaluate an ensemble of <number_of_models> models with test-split <split> threshold "<threshold>" and log evaluation in "<output>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the train dataset has been created
                And I check that the test dataset has been created
                And I check that the ensemble has been created
                And I check that the evaluation has been created
                Then the evaluation key "<key>" value for the model is greater than <value>
                And I evaluate the ensemble in directory "<directory>" with the dataset in directory "<directory>" and options "<options>" and log evaluation in "<output2>"
                And I check that the evaluation has been created
                Then the evaluation key "<key>" value for the model is greater than <value>
        """
        print(self.test_scenario9.__doc__)
        headers = ["data", "output", "split", "number_of_models", "key",
                   "value", "directory", "output2", "options"]
        examples = [
            ['data/iris.csv', 'scenario_e9/evaluation', '0.2', '5',
             'average_phi', '0.89', 'scenario_e9', 'scenario_e9/evaluation',
             "--method threshold --threshold 5 --class Iris-virginica"]]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            evaluation.i_create_with_split_to_evaluate_ensemble(
                self, data=example["data"],
                number_of_models=example["number_of_models"],
                split=example["split"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_dataset(self, suffix='train ')
            test_pred.i_check_create_dataset(self, suffix='test ')
            test_pred.i_check_create_ensemble(self)
            test_pred.i_check_create_evaluation(self)
            evaluation.i_check_evaluation_key(
                self, key=example["key"], value=example["value"])
            evaluation.i_evaluate_ensemble_with_dataset_and_options(
                self, ensemble_dir=example["directory"],
                dataset_dir=example["directory"],
                output=example["output2"], options=example["options"])
            test_pred.i_check_create_evaluation(self)
            evaluation.i_check_evaluation_key(
                self, key=example["key"], value=example["value"])

    def test_scenario10(self):
        """
            Scenario: Successfully building evaluations for time series from start:
                Given I create BigML time series resources uploading train "<data>" file to evaluate and log evaluation in "<output>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the time series has been created
                And I check that the evaluation has been created
                Then the evaluation file is like "<json_evaluation_file>"
        """
        print(self.test_scenario10.__doc__)
        headers = ["data", "output", "json_evaluation_file"]
        examples = [
            ['data/grades.csv', 'scenario_e10/evaluation',
             'check_files/evaluation_grades_ts.json']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            ts_pred.i_create_all_ts_resources_to_evaluate(
                self, data=example["data"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            ts_pred.i_check_create_time_series(self)
            test_pred.i_check_create_evaluation(self)
            evaluation.then_the_evaluation_file_is_like(
                self, example["json_evaluation_file"])

    def test_scenario11(self):
        """
            Scenario: Successfully building evaluations for deepnets from start:
                Given I create BigML deepnet resources uploading train "<data>" file to evaluate and log evaluation in "<output>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the deepnet has been created
                And I check that the evaluation has been created
                Then the evaluation file is like "<json_evaluation_file>"
        """
        print(self.test_scenario11.__doc__)
        headers = ["data", "output", "json_evaluation_file"]
        examples = [
            ['data/iris.csv', 'scenario_e11/evaluation',
             'check_files/evaluation_iris_dn.json']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            dn_pred.i_create_all_dn_resources_to_evaluate(
                self, data=example["data"], output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            dn_pred.i_check_create_dn_model(self)
            test_pred.i_check_create_evaluation(self)
            evaluation.then_the_evaluation_file_is_like(
                self, example["json_evaluation_file"])



# -*- coding: utf-8 -*-
#
# Copyright 2020-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Resources management functions

"""


import sys

import bigml.api

from bigmler.utils import (dated, get_url, log_message, check_resource,
                           check_resource_error, log_created_resources)

from bigmler.resourcesapi.common import set_basic_args, \
    update_attributes



def set_library_args(args, name=None):
    """Returns a library arguments dict

    """

    if name is None:
        name = args.name
    library_args = set_basic_args(args, name)
    if args.project_id is not None:
        library_args.update({"project": args.project_id})
    if args.imports is not None:
        library_args.update({"imports": args.imports_})
    update_attributes(library_args, args.json_args.get('library'))
    return library_args


def create_library(source_code, library_args, args, api=None, path=None,
                   session_file=None, log=None):
    """Creates remote library

    """
    if api is None:
        api = bigml.api.BigML()

    message = dated("Creating library \"%s\".\n" % library_args["name"])
    log_message(message, log_file=session_file, console=args.verbosity)
    library = api.create_library(source_code, library_args)
    log_created_resources("library", path,
                          bigml.api.get_library_id(library), mode='a')
    library_id = check_resource_error(library, "Failed to create library: ")
    try:
        library = check_resource(library, api.get_library, raise_on_error=True)
    except Exception as exception:
        sys.exit("Failed to get a compiled library: %s" % str(exception))
    message = dated("Library created: %s\n" % get_url(library))
    log_message(message, log_file=session_file, console=args.verbosity)
    log_message("%s\n" % library_id, log_file=log)
    return library



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2015-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing clusters

"""
from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)


import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.basic_cluster_prediction_steps as test_cluster


def setup_module():
    """Setup for the module

    """
    common_setup_module()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()

class TestCluster:
    """Testing cluster command """

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def test_scenario1(self):
        """
        Scenario: Successfully building test centroids from scratch:
            Given I create BigML resources uploading train "<data>" file to create centroids for "<test>" and log predictions in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the cluster has been created
            And I check that the centroids are ready
            Then the local centroids file is like "<predictions_file>"
        """
        print(self.test_scenario1.__doc__)
        headers = ["data", "test", "output", "predictions_file"]
        examples = [
            ['data/grades.csv', 'data/grades.csv',
             'scenario_c_1_r/centroids.csv',
             'check_files/centroids_grades.csv'],
            ['data/diabetes.csv', 'data/diabetes.csv',
             'scenario_c_1/centroids.csv',
             'check_files/centroids_diabetes.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_cluster.i_create_all_cluster_resources(
                self, data=example["data"], test=example["test"],
                output=example["output"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_cluster(self)
            test_cluster.i_check_create_centroids(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario2(self):
        """
        Scenario: Successfully building test predictions from source
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using source to find centroids for "<test>" and log predictions in "<output>"
            And I check that the dataset has been created
            And I check that the cluster has been created
            And I check that the centroids are ready
            Then the local centroids file is like "<predictions_file>"
        """
        print(self.test_scenario2.__doc__)
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario_c_1', '{"data": "data/diabetes.csv",' +
             ' "output": "scenario_c_1/centroids.csv",' +
             ' "test": "data/diabetes.csv"}', 'data/diabetes.csv',
             'scenario_c_2/centroids.csv',
             'check_files/centroids_diabetes.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_cluster.i_create_cluster_resources_from_source(
                self, test=example["test"], output=example["output"])
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_cluster(self)
            test_cluster.i_check_create_centroids(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario3(self):
        """
        Scenario: Successfully building test predictions from dataset
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using dataset to find centroids for "<test>" and log predictions in "<output>"
            And I check that the cluster has been created
            And I check that the centroids are ready
            Then the local centroids file is like "<predictions_file>"
        """
        print(self.test_scenario3.__doc__)
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario_c_1', '{"data": "data/diabetes.csv",' +
             ' "output": "scenario_c_1/centroids.csv",' +
             ' "test": "data/diabetes.csv"}', 'data/diabetes.csv',
             'scenario_c_3/centroids.csv',
             'check_files/centroids_diabetes.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_cluster.i_create_cluster_resources_from_dataset(
                self, test=example["test"], output=example["output"])
            test_pred.i_check_create_cluster(self)
            test_cluster.i_check_create_centroids(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario4(self):
        """
        Scenario: Successfully building test predictions from cluster
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using cluster to find centroids for "<test>" and log predictions in "<output>"
            And I check that the centroids are ready
            Then the local centroids file is like "<predictions_file>"
        """
        print(self.test_scenario4.__doc__)
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario_c_1', '{"data": "data/diabetes.csv",' +
             ' "output": "scenario_c_1/centroids.csv",' +
             ' "test": "data/diabetes.csv"}', 'data/diabetes.csv',
             'scenario_c_4/centroids.csv',
             'check_files/centroids_diabetes.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_cluster.i_create_cluster_resources_from_cluster(
                self, test=example["test"], output=example["output"])
            test_cluster.i_check_create_centroids(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario5(self):
        """
        Scenario: Successfully building test predictions from clusters file
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using clusters in file "<clusters_file>" to find centroids for "<test>" and log predictions in "<output>"
            And I check that the centroids are ready
            Then the local centroids file is like "<predictions_file>"
        """
        print(self.test_scenario5.__doc__)
        headers = ["scenario", "kwargs", "clusters_file", "test", "output",
                   "predictions_file"]
        examples = [
            ['scenario_c_1', '{"data": "data/diabetes.csv", "output": "scenario_c_1/centroids.csv", "test": "data/diabetes.csv"}', 'scenario_c_1/clusters', 'data/diabetes.csv', 'scenario_c_5/centroids.csv', 'check_files/centroids_diabetes.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_cluster.i_create_cluster_resources_from_clusters_file(
                self, clusters_file=example["clusters_file"],
                test=example["test"], output=example["output"])
            test_cluster.i_check_create_centroids(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario6(self):
        """
        Scenario: Successfully generating datasets from cluster centroids
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I generate datasets for "<centroid_names>" centroids and log predictions in "<output>"
            Then I check that the <datasets_number> cluster datasets are ready
        """
        print(self.test_scenario6.__doc__)
        headers = ["scenario", "kwargs", "centroid_names", "output",
                   "datasets_number"]
        examples = [
            ['scenario_c_1', '{"data": "data/diabetes.csv",' +
             ' "output": "scenario_c_1/centroids.csv",' +
             ' "test": "data/diabetes.csv"}', 'Cluster 1,Cluster 2',
             'scenario_c_6/centroids.csv', '2']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_cluster.i_create_datasets_from_cluster(
                self, centroids=example["centroid_names"],
                output=example["output"])
            test_cluster.i_check_cluster_datasets(
                self, datasets_number=example["datasets_number"])

    def test_scenario7(self):
        """
        Scenario: Successfully building test predictions from cluster
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML resources using local cluster in "<scenario>" to find centroids for "<test>" and log predictions in "<output>"
            And I check that the centroids are ready
            Then the local centroids file is like "<predictions_file>"
        """
        print(self.test_scenario7.__doc__)
        headers = ["scenario", "kwargs", "test", "output", "predictions_file"]
        examples = [
            ['scenario_c_1', '{"data": "data/diabetes.csv", "output": "scenario_c_1/centroids.csv", "test": "data/diabetes.csv"}', 'data/diabetes.csv', 'scenario_c_7/centroids.csv', 'check_files/centroids_diabetes.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_cluster.i_create_cluster_resources_from_local_cluster(
                self, directory=example["scenario"], test=example["test"],
                output=example["output"])
            test_cluster.i_check_create_centroids(self)
            test_pred.i_check_predictions(self, example["predictions_file"])

    def test_scenario8(self):
        """
        Scenario: Successfully generating models from cluster centroids
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I generate models for "<centroid_names>" centroids and log results in "<output>"
            Then I check that the <model_number> cluster models are ready
        """
        print(self.test_scenario8.__doc__)
        headers = ["scenario", "kwargs", "centroid_names", "output",
                   "models_number"]
        examples = [
            ['scenario_c_1', '{"data": "data/diabetes.csv",' +
             ' "output": "scenario_c_1/centroids.csv",' +
             ' "test": "data/diabetes.csv"}', 'Cluster 1,Cluster 2',
             'scenario_c_8/centroids.csv', '2']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_cluster.i_create_models_from_cluster(
                self, centroids=example["centroid_names"],
                output=example["output"])
            test_cluster.i_check_create_cluster(self)
            test_cluster.i_check_cluster_models(
                self, models_number=example["models_number"])

    def test_scenario9(self):
        """
        Scenario: Successfully building test predictions from dataset with summary fields
            Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
            And I create BigML cluster using dataset and summary fields "<summary_fields>" and log resources in "<output_dir>"
            And I check that the cluster has been created
            Then the cluster has summary fields "<summary_fields>"
        """
        print(self.test_scenario9.__doc__)
        headers = ["scenario", "kwargs", "output_dir", "summary_fields"]
        examples = [
            ['scenario_c_1', '{"data": "data/diabetes.csv",' +
             ' "output": "scenario_c_1/centroids.csv",' +
             ' "test": "data/diabetes.csv"}', 'scenario_c_9', '000008,000007']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_have_previous_scenario_or_reproduce_it(
                self, example["scenario"], example["kwargs"])
            test_cluster.i_create_cluster_from_dataset_with_summary_fields(
                self, summary_fields=example["summary_fields"],
                output_dir=example["output_dir"])
            test_pred.i_check_create_cluster(self)
            test_cluster.i_check_cluster_has_summary_fields(
                self, example["summary_fields"])



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-import
#
# Copyright 2020-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Common Resource management functions

"""


import sys
import time

try:
    import simplejson as json
except ImportError:
    import json

import bigml.api

from bigml.util import bigml_locale
from bigml.multivote import THRESHOLD_CODE
from bigml.constants import EXTERNAL_CONNECTION_ATTRS
from bigml.api_handlers.externalconnectorhandler import get_env_connection_info

from bigmler.utils import (dated, get_url, log_message, plural, check_resource,
                           check_resource_error, log_created_resources,
                           transform_fields_keys, is_shared, FILE_ENCODING)
from bigmler.labels import label_model_args, get_all_labels
from bigmler.reports import report


EVALUATE_SAMPLE_RATE = 0.8
SEED = "BigML, Machine Learning made easy"
LOCALE_DEFAULT = "en_US"
FIELDS_QS = 'only_model=true'
ALL_FIELDS_QS = "limit=-1"
ADD_PREFIX = '+'
REMOVE_PREFIX = '-'
ADD_REMOVE_PREFIX = [ADD_PREFIX, REMOVE_PREFIX]
BRIEF_FORMAT = 'brief'
NORMAL_FORMAT = 'normal'
FULL_FORMAT = 'full'
VALID_FIELD_ATTRIBUTES = {
    "source": ["name", "label", "description", "optype", "term_analysis"],
    "dataset": ["name", "label", "description", "preferred", "term_analysis"]}
BOOSTING_OPTIONS = ["iterations", "early_holdout", "learning_rate", \
    "early_out_of_bag", "step_out_of_bag"]
DS_NAMES = "ABCDEFGHIJKLMNOPQRSTUVXYZ"


def get_basic_seed(order):
    """ Builds a standard seed from a text adding the order

    """
    return "%s - %s" % (SEED, order)


def shared_changed(shared, resource):
    """Returns True if the shared status of the resource differs from the user
       given value

    """
    return is_shared(resource) != shared


def configure_input_fields(fields, user_given_fields, by_name=False):
    """ Returns the input fields used in the new resource creation as given

        The user can choose to write all the fields that will be used in the
        new resource or modify the set of fields retrieved from the
        resource that will be used to create the new one.
    """
    def modify_input_fields(prefix, field, input_fields):
        """Adds or removes according to the prefix in the given field
           this field from the list of input fields.

        """
        if prefix == ADD_PREFIX:
            if not field in input_fields:
                input_fields.append(field)
        elif field in input_fields:
            input_fields.remove(field)

    # case of adding and removing fields to the dataset preferred field set
    if all(name[0] in ADD_REMOVE_PREFIX for name in user_given_fields):
        preferred_fields = fields.preferred_fields()
        input_fields = list(preferred_fields.keys())
        if by_name:
            input_fields = [fields.field_name(field_id) for field_id in
                            input_fields]
        for name in user_given_fields:
            prefix = name[0]
            field_name = name[1:]
            if by_name:
                modify_input_fields(prefix, field_name, input_fields)
            else:
                try:
                    field_id = fields.field_id(field_name)
                except ValueError as exc:
                    sys.exit(exc)
                modify_input_fields(prefix, field_id, input_fields)
    # case of user given entire list of fields
    else:
        if by_name:
            return user_given_fields
        input_fields = []
        for name in user_given_fields:
            try:
                input_fields.append(fields.field_id(name))
            except ValueError as exc:
                sys.exit(exc)
    return input_fields


def utf8(text):
    """Encodes using the global FILE_ENCODING

    """
    return text.encode(FILE_ENCODING)

def update_attributes(updatable_attributes, new_attributes, by_column=False,
                      fields=None):
    """Correctly merging the "fields" attribute substructure in updates.

       updatable_attributes: previous attributes to be updated
       new_attributes: updates to be added
       by_column: set to True is keys are the column position of the field
       fields: Fields object info
    """
    if new_attributes:
        fields_substructure = updatable_attributes.get("fields", {})
        field_attributes = new_attributes.get("fields", {})
        if field_attributes and (not by_column or fields):
            for field_key, value in list(field_attributes.items()):
                field_id = (field_key if not by_column
                            else fields.field_id(field_key))
                if not field_id in list(fields_substructure.keys()):
                    fields_substructure.update({field_id: {}})
                fields_substructure[field_id].update(value)
            updatable_attributes.update({"fields": fields_substructure})
        else:
            updatable_attributes.update(new_attributes)


def update_json_args(resource_attributes, json_attributes, fields=None):
    """Updating the resource attributes with the contents of a JSON file

    """
    if fields is not None:
        # transforms the fields structure changes if columns are used as keys
        json_attributes = transform_fields_keys(json_attributes, fields)
    update_attributes(resource_attributes, json_attributes)


def relative_input_fields(fields, user_given_fields):
    """Returns the user given input fields using relative syntax

    """

    input_fields = []
    if all((name[0] in ADD_REMOVE_PREFIX) for name in user_given_fields):
        return user_given_fields

    preferred_fields = fields.preferred_fields()
    for field_id in list(preferred_fields.keys()):
        name = fields.fields[field_id]['name']
        if not name in user_given_fields:
            input_fields.append("%s%s" % (REMOVE_PREFIX, name))
    for name in user_given_fields:
        try:
            field_id = fields.field_id(name)
        except ValueError as exc:
            sys.exit(exc)
        input_fields.append("%s%s" % (ADD_PREFIX, name))

    return input_fields


def wait_for_available_tasks(inprogress, max_parallel, api,
                             resource_type, wait_step=2):
    """According to the max_parallel number of parallel resources to be
       created, when the number of in progress resources reaches the limit,
       it checks the ones in inprogress to see if there's a
       FINISHED or FAULTY resource. If found, it is removed from the
       inprogress list and returns to allow another one to be created.

    """

    check_kwargs = {"retries": 0, "query_string": "full=false", "api": api}
    while len(inprogress) == max_parallel:
        for j, resource_id in enumerate(inprogress):
            try:
                ready = check_resource(resource_id, **check_kwargs)
                status = bigml.api.get_status(ready)
                if status['code'] == bigml.api.FINISHED:
                    del inprogress[j]
                    return
                if status['code'] == bigml.api.FAULTY:
                    raise ValueError(status['message'])
            except ValueError as exception:
                sys.exit("Failed to get a finished %s: %s" %
                         (resource_type, str(exception)))
        time.sleep(max_parallel * wait_step)


def check_fields_struct(update_args, resource_type):
    """In case the args to update have a `fields` attribute, it checks the
    structure in this attribute and removes the attributes for each field
    that will not be accepted by the API.
    """
    if "fields" in update_args:
        fields_substr = update_args.get("fields")
        for _, field in list(fields_substr.items()):
            attributes = list(field.keys())
            for attribute in attributes:
                if not attribute in VALID_FIELD_ATTRIBUTES.get(resource_type):
                    del field[attribute]


def set_basic_args(args, name):
    """Sets the basic arguments, common to all resources

    """
    basic = {
        "description": args.description_,
        "category": args.category,
        "tags": args.tag}
    if name is not None:
        basic.update({"name": name})
    return basic


def set_basic_model_args(args, name):
    """Sets the additional args common to all models

    """
    model_args = set_basic_args(args, name)
    if args.default_numeric_value is not None:
        model_args.update({ \
            "default_numeric_value": args.default_numeric_value})
    return model_args


def set_basic_batch_args(args, name):
    """Sets the additional args common to all batch resources

    """
    batch_args = set_basic_args(args, name)
    header = (hasattr(args, "prediction_header") and args.prediction_header) \
        or (hasattr(args, "projection_header") and args.projection_header)
    batch_args.update({ \
        "header": header,
        "output_dataset": args.to_dataset
    })
    return batch_args


def map_fields(fields_map, model_fields, dataset_fields):
    """Build a dict to map model to dataset fields

    """
    update_map = {}
    for (model_column, dataset_column) in fields_map.items():
        try:
            update_map.update({
                model_fields.field_id(model_column):
                dataset_fields.field_id(dataset_column)})
        except ValueError as exc:
            sys.exit(exc)

    return update_map


def update_sample_parameters_args(resource_args, args):
    """Updates the information related to the common sampling options

    """
    if args.sample_rate != 1:
        resource_args.update({"sample_rate": args.sample_rate})
        if hasattr(args, "out_of_bag") and args.out_of_bag:
            resource_args.update({"out_of_bag": True})
    if hasattr(args, "replacement") and args.replacement:
        resource_args.update({"replacement": True})
    if hasattr(args, "randomize") and args.randomize:
        resource_args.update({"randomize": True})
    return resource_args


def save_txt_and_json(object_dict, output, api=None):
    """Saves in txt and JSON format the contents of a dict object

    """
    open_mode = 'wt'
    message = json.dumps(object_dict)
    with open(output + '.json', open_mode) as dict_json:
        dict_json.write(message)
    with open(output + '.txt', open_mode) as dict_txt:
        api.pprint(object_dict, dict_txt)


def hasattr_(obj, attr):
    """Checking if an object has a non-empty attribute """
    return hasattr(obj, attr) and (
        (isinstance(getattr(obj, attr), (list, dict)) and
         getattr(obj, attr))
        or
        (not isinstance(getattr(obj, attr), (list, dict)) and
         getattr(obj, attr) is not None))


def set_config_attrs(args, attrs, config, attr_aliases=None, exclusive=False):
    """Checks that the attributes have been set as a bigmler option
    and not empty before adding them as
    a resource configuration value. If the attribute in the configuration
    has a different name, that should be provided in the `attrs_alias`
    argument. If `exclusive` is provided, when the process ends when the first
    non-empty attribute is set.
    """
    for attr in attrs:
        if hasattr_(args, attr):
            if not isinstance(attr_aliases, dict) or \
                    attr_aliases.get(attr) is None:
                config_attr = attr
            else:
                config_attr = attr_aliases.get(attr)
            config[config_attr] = getattr(args, attr)
            if exclusive:
                break



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2015-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing weight options

"""
from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)


import bigmler.tests.basic_tst_prediction_steps as test_pred


def setup_module():
    """Setup for the module

    """
    common_setup_module()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()

class TestWeights:
    """Testing weights options in command"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def test_scenario1(self):
        """
            Scenario: Successfully building a balanced model
                Given I create a BigML balanced model from "<data>" and store logs in "<output_dir>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the model has been created
                Then I check that the model is balanced
        """
        print(self.test_scenario1.__doc__)
        headers = ["data", "output_dir"]
        examples = [
            ['data/iris.csv', 'scenario_w_1']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_balanced_model(
                self, data=example["data"], output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_model(self)
            test_pred.i_check_model_is_balanced(self)

    def test_scenario2(self):
        """
            Scenario: Successfully building a field weighted model
                Given I create a BigML field weighted model from "<data>" using field "<field>" as weight and "<objective>" and store logs in "<output_dir>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the model has been created
                Then I check that the model uses as weight "<field_id>"
        """
        print(self.test_scenario2.__doc__)
        headers = ["data", "field", "output_dir", "field_id", "objective"]
        examples = [
            ['data/iris_w.csv', 'weight', 'scenario_w_2', '000005', 'species']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_weighted_field_model(
                self, data=example["data"], field=example["field"],
                output_dir=example["output_dir"],
                objective=example["objective"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_model(self)
            test_pred.i_check_weighted_model(
                self, field=example["field_id"])

    def test_scenario3(self):
        """
            Scenario: Successfully building a objective weighted model
                Given I create a BigML objective weighted model from "<data>" using the objective weights in file "<path>" and store logs in "<output_dir>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the model has been created
                Then I check that the model uses as objective weights "<weights>"

                Examples:
                |data |path | output_dir  | weights
                |../data/iris.csv | ../data/weights.csv |./scenario_w_3 | [["Iris-setosa",5], ["Iris-versicolor",3]]
        """
        print(self.test_scenario3.__doc__)
        headers = ["data", "path", "output_dir", "weights"]
        examples = [
            ['data/iris.csv', 'data/weights.csv', 'scenario_w_3',
             '[["Iris-setosa",5], ["Iris-versicolor",3]]']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_pred.i_create_objective_weighted_model(
                self, data=example["data"], path=example["path"],
                output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_model(self)
            test_pred.i_check_objective_weighted_model(
                self, weights=example["weights"])



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2014-2023 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os
import json

from bigml.api import check_resource

from bigmler.utils import storage_file_name
from bigmler.tests.common_steps import shell_execute, check_rows_equal
from bigmler.tests.world import world, res_filename, ok_, eq_


def i_create_all_anomaly_resources_without_test_split(
    step, data=None, output_dir=None):
    """Step: I create BigML resources uploading train <data> file to find
    anomaly scores for the training set remotely saved to dataset with no
    CSV output and log resources in <output_dir>
    """
    ok_(data is not None and output_dir is not None)
    command = ("bigmler anomaly --remote --train " + res_filename(data) +
               " --store --score --no-csv --to-dataset --output-dir " +
               output_dir)
    shell_execute(command, "%s/x.csv" % output_dir, data=data, test_split=1)


def i_create_all_anomaly_resources_with_test_split_no_csv(
    step, data=None, test_split=None, output_dir=None):
    """Step: I create BigML resources uploading train <data> file to find
    anomaly scores with test split <test_split> remotely saved to dataset
    with no CSV output and log resources in <output_dir>
    """
    ok_(data is not None and output_dir is not None)
    command = ("bigmler anomaly --remote --train " + res_filename(data) +
               " --test-split " + test_split +
               " --store --no-csv --prediction-header  "
               "--to-dataset --output-dir " + output_dir)
    shell_execute(command, "%s/x.csv" % output_dir,
                  data=data, test_split=test_split)


def i_create_anomaly_with_id_fields(step, id_fields=None, output_dir=None):
    """Step: And I create anomaly detector with "<id_fields>" and log
    predictions in <output_dir>
    """
    ok_(output_dir is not None and id_fields is not None)
    command = ("bigmler anomaly --dataset " + world.dataset["resource"] +
               " --id-fields " + ",".join(json.loads(id_fields)) +
               " --store --output-dir " + output_dir)
    shell_execute(command, "%s/x.csv" % output_dir)


def i_create_anomaly_resources_with_options(
    step, data=None, options=None, output_dir=None):
    """Step: I create BigML anomaly detector from data <data> with options
    <options> and generate a new dataset of anomalies in <output_dir>
    """
    ok_(data is not None and output_dir is not None and options is not None)
    command = ("bigmler anomaly --train " + res_filename(data) + " " +
               options +
               " --anomalies-dataset in --store --output-dir " + output_dir)
    shell_execute(command, "%s/x.csv" % output_dir, data=data)


def i_create_all_anomaly_resources(step, data=None, test=None, output=None):
    """Step: I create BigML resources uploading train <data> file to create
    anomaly scores for <test> and log predictions in <output>
    """
    ok_(data is not None and test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler anomaly --train " + res_filename(data) + " --test " +
               test + " --store --prediction-header --output " + output)
    shell_execute(command, output, test=test)


def i_create_anomaly_resources_from_source(step, test=None, output=None):
    """Step: I create BigML resources using source to find anomaly scores for
    <test> and log predictions in <output>
    """
    ok_(test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler anomaly --source " + world.source['resource'] +
               " --test " + test +
               " --store --prediction-header --output " + output)
    shell_execute(command, output, test=test)


def i_create_anomaly_resources_from_dataset(step, test=None, output=None):
    """Step: I create BigML resources using dataset to find anomaly scores
    for <test> and log predictions in <output>
    """
    ok_(test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler anomaly --dataset " + world.dataset['resource'] +
               " --test " + test +
               " --store --prediction-header --output " + output)
    shell_execute(command, output, test=test)


def i_create_anomaly_resources_from_local_anomaly_detector(
    step, directory=None, test=None, output=None):
    """Step: I create BigML resources using local anomaly detector in
    <directory> to find anomaly scores for <test> and log predictions in
    <output>
    """
    ok_(test is not None and output is not None and directory is not None)
    test = res_filename(test)
    with open(os.path.join(directory, "anomalies")) as handler:
        anomaly_id = handler.readline().strip()
    command = ("bigmler anomaly --anomaly-file " +
               storage_file_name(directory, anomaly_id) +
               " --test " + test +
               " --store --prediction-header --output " + output)
    shell_execute(command, output, test=test)


def i_create_anomaly_resources_from_anomaly_detector(
    step, test=None, output=None):
    """Step: I create BigML resources using anomaly detector to find anomaly
    scores for <test> and log predictions in <output>
    """
    ok_(test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler anomaly --anomaly " + world.anomaly['resource'] +
               " --test " + test +
               " --store --prediction-header --output " + output)
    shell_execute(command, output, test=test)


def i_create_anomaly_resources_from_anomaly_file(
    step, anomaly_file=None, test=None, output=None):
    """Step: I create BigML resources using anomaly detector in file
    <anomaly_file> to find anomaly scores for <test> and log predictions in
    <output>
    """
    ok_(anomaly_file is not None and test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler anomaly --anomalies " + anomaly_file +
               " --test " + test +
               " --store --prediction-header --output " + output)
    shell_execute(command, output, test=test)


def i_check_create_anomaly(step):
    """Step: I check that the anomaly detector has been created"""
    anomaly_file = "%s%sanomalies" % (world.directory, os.sep)
    message = None
    try:
        with open(anomaly_file) as handler:
            anomaly = check_resource(handler.readline().strip(),
                                     api=world.api)
        world.anomalies.append(anomaly['resource'])
        world.anomaly = anomaly
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_create_anomaly_scores(step):
    """Step: I check that the anomaly scores are ready"""
    predictions_lines = 0
    message = None
    try:
        predictions_file = world.output
        with open(predictions_file) as handler:
            predictions_lines = 0
            for _ in handler:
                predictions_lines += 1
            if predictions_lines != world.test_lines:
                message = "predictions lines: %s, test lines: %s" % (
                    predictions_lines, world.test_lines)
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_anomaly_scores(step, check_file):
    """Step: the local anomaly scores file is like <check_file>"""
    check_file = res_filename(check_file)
    predictions_file = world.output
    message = None
    try:
        message = check_rows_equal(predictions_file, check_file)
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_anomaly_has_id_fields(step, id_fields=None):
    """Checking that the anomaly detector has the correct id fields"""
    ok_(id_fields is not None)
    id_fields = json.loads(id_fields)
    id_fields.sort()
    anomaly_id_fields = world.anomaly.get("object", {}).get("id_fields")
    anomaly_id_fields.sort()
    eq_(len(anomaly_id_fields), len(id_fields))
    eq_(anomaly_id_fields, id_fields)


def i_create_all_anomaly_resources_with_mapping(
    step, data=None, test=None, fields_map=None, output=None):
    """Step: I create BigML resources uploading train <data> file to find
    anomaly scores for <test> remotely with mapping file <fields_map> and log
    predictions in <output>
    """
    ok_(data is not None and test is not None and output is not None
        and fields_map is not None)
    test = res_filename(test)
    command = ("bigmler anomaly --remote --train " + res_filename(data) +
               " --test " + test +
               " --fields-map " + res_filename(fields_map) +
               " --store --prediction-header --output " + output)
    shell_execute(command, output, test=test)


def i_create_all_anomaly_resources_with_test_split(
    step, data=None, test_split=None, output=None):
    """Step: I create BigML resources uploading train <data> file to find
    anomaly scores with test split <test_split> remotely and log predictions
    in <output>
    """
    ok_(data is not None and output is not None and test_split is not None)
    data = res_filename(data)
    command = ("bigmler anomaly --remote --train " + data +
               " --test-split " + test_split +
               " --store --output " + output)
    shell_execute(command, output, data=data, test_split=test_split)


def i_check_no_local_csv(step):
    """Step: no local CSV file is created"""
    files = [file_name for file_name in os.listdir(world.directory)
             if file_name.endswith('.csv')]
    eq_(len(files), 0, msg="Found unexpected files: %s" % ", ".join(files))


def i_check_create_batch_anomaly_score_dataset(step):
    """Step: I check that the batch anomaly scores dataset exists"""
    dataset_file = os.path.join(world.directory, "batch_anomaly_score_dataset")
    message = None
    try:
        with open(dataset_file) as handler:
            dataset = check_resource(handler.readline().strip(),
                                     api=world.api)
        world.datasets.append(dataset['resource'])
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def i_check_top_anomalies(step, top_anomalies):
    """Step: the top anomalies in the anomaly detector are <top_anomalies>"""
    eq_(int(top_anomalies),
        len(world.anomaly.get('object', []).get(
            'model', []).get('top_anomalies')))


def i_check_forest_size(step, forest_size):
    """Step: the forest size in the anomaly detector is <forest_size>"""
    eq_(int(forest_size),
        world.anomaly.get('object', []).get('forest_size'))


def i_check_dataset_lines_number(step, lines_number):
    """Step: the number of records in the top anomalies dataset is
    <top_anomalies>
    """
    eq_(int(lines_number),
        world.dataset.get('object', []).get('rows'))
